﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/
package net.flash_line.as3.util.common {	
	import flash.display.* ;
	import flash.text.TextField ;
	import flash.text.TextFormat ;
	import flash.text.TextFieldAutoSize ;
	import flash.text.Font;
	import flash.text.TextFieldType ;
	import flash.events.* ;		
	import net.flash_line.as3.util.global.* ;		
	/**
	* Manage a unique alert runtime window - used to debug only
	* <p>usage : alert("[my debug message] myVar="+myVar); </p>
	* @see SpriteX#alert()
	* @see Super#alert()
	*/ 
	public class AlertWin  {	
		static 	private var pX		: Number ; 	// window pos x 
		static 	private var pY 		: Number ; 	// window pos y 
		static 	private var pW		: Number ; 	// window width 
		static 	private var pH 		: Number ; 	// window height 
		static 	private var pHBg	: Number ; 	// window head background color
		static 	private var pHTx 	: Number ; 	// window head text color
		static 	private var pBg 	: Number ; 	// window  background color
		static 	private var pTx 	: Number ; 	// window  text color
		static 	private var	pTitle	: String ; 	// window title 
		static 	private var	pTF		: TextField ; 	// body textField
		static 	private var	pClose	: SpriteX ; 	// close button
		static 	private var	pHead	: SpriteX ; 	// header sprite
		static 	private var	pFmt	: TextFormat ; 	// text format
		static 	private var	pCtnr	: DisplayObjectContainer ; // the stage;
		static 	private var	pWin	: SpriteX ; // alert windows			
		/**
		* Create AlertWin in a choosed DisplayObjectContainer
		* <p>This method must be called one time at start of application</p>
		* <p>see net.flash_line.as3.util.global.appInit : Function</p>
		* @param	p The container of visible alert window
		*/	
		static 	public function create (p:DisplayObjectContainer=null) {
			if (pCtnr==null) {
				pCtnr=p;
			}			
		}
		/**
		* Set window's parameters
		* @see #editWin()
		* @param	x		pos x
		* @param	y		pos y
		* @param	w		width
		* @param	h		height
		* @param	title	top title
		* @param	hBg		Header background color 
		* @param	hTx		Header text
		* @param	bg		background color 
		* @param	tx		Message text color
		*/
		static 	public function setWin (x:Number=100,y:Number=100,w:Number=450,h:Number=400,title:String="Alert",hBg:Number=0xFF3300,hTx:Number=0xFFFFFF,bg:Number=0xdddddd ,tx:Number=0x000000) {
			if (pTitle==null) {	
				pX=x;
				pY=y;
				pW=w;
				pH=h;
				pTitle=title;
				pHBg=hBg;
				pHTx=hTx;
				pBg=bg;
				pTx=tx;
			}
		}
		/**
		* Edit window's parameters
		* @see #setWin()
		* @param	x		pos x
		* @param	y		pos y
		* @param	w		width
		* @param	h		height
		* @param	title	top title
		* @param	hBg		Header background color 
		* @param	hTx		Header text
		* @param	bg		background color
		* @param	tx		Message text color
		*/
		static 	public function editWin (x:Number=100,y:Number=100,w:Number=450,h:Number=400,title:String="Alert",hBg:Number=0xFF3300,hTx:Number=0xFFFFFF,bg:Number=0xdddddd ,tx:Number=0x000000) {
				pX=x;
				pY=y;
				pW=w;
				pH=h;
				pTitle=title;
				pHBg=hBg;
				pHTx=hTx;
				pBg=bg;
				pTx=tx;
		}
		/**
		* display message 
		* @param	msg message's  text
		* @return	true if okay
		*/
		static 	public function display (msg : String ):Boolean {	
			if (openWin()==false) return false ;
			pTF.appendText("> "+msg+"\n");
			pTF.setTextFormat(pFmt);
			return true;
			
		}
		/**
		* Open the window
		* @return true if okay
		*/
		static private function openWin():Boolean{
			if (pCtnr==null) return false ;
			if (pWin!=null) return true ;
			pWin=new SpriteX();
			pCtnr.addChild(pWin);
			//head
			var head_mc:SpriteX=new SpriteX();
			pWin.addChild(head_mc);
			head_mc.graphics.beginFill(pHBg);
			var hh:Number=30; // Head height
			head_mc.graphics.drawRect(0, 0,pW,hh);
			/// head text
			var title_tf:TextField = new TextField();
            title_tf.autoSize = TextFieldAutoSize.LEFT;
            title_tf.background = true;
			title_tf.backgroundColor = pHBg ;
			title_tf.textColor = pHTx ;
			var fmt:TextFormat=new TextFormat();
			fmt.size=20; 			
			if (isStrSet(Var.font)) fmt.font = Var.font; 
			else fmt.font = "Arial";	
			fmt.bold=true;
			title_tf.text=pTitle;
			title_tf.setTextFormat(fmt);			
			var t_mc:Sprite=new Sprite();
			t_mc.addChild(title_tf);
			head_mc.addChild(t_mc);
			t_mc.x=(head_mc.width-t_mc.width)/2;
			t_mc.y=head_mc.y;
			pHead=head_mc;
			//
			// close button
			var b_mc:SpriteX=new SpriteX();
			b_mc.graphics.lineStyle(2,0);
			var size:Number=hh/2;
			b_mc.graphics.drawRect(0, 0,size,size);
			head_mc.addChild(b_mc);
			b_mc.x=pW-size-3;
			b_mc.y=(hh-size)/2;
			var tf:TextField =new TextField();
            tf.background = false;
			tf.textColor = 0 ;
			tf.y-=3;
			fmt=new TextFormat();
			fmt.size=16; 
			fmt.font = "Arial";
			fmt.bold=true;
			tf.text="X";
			tf.setTextFormat(fmt);			
			b_mc.addChild(tf);
			pClose=b_mc;
			createEvent();
			//
			//body_mc			
			var body_mc:SpriteX=new SpriteX();
			body_mc.graphics.beginFill(pBg);
			body_mc.graphics.drawRect(0,0,pW,pH-hh);
			pWin.addChild(body_mc);
			body_mc.y=hh;
			/// body_mc text
			tf = new TextField();
            //tf.autoSize = TextFieldAutoSize.LEFT;
			tf.width=pW*.95;
			tf.height=pH-hh;
            tf.background = true;
			tf.backgroundColor = pBg ;
			tf.textColor = pTx ;
			tf.selectable=true;
			tf.multiline=true;
			tf.wordWrap=true;
			tf.type=TextFieldType.INPUT;
			fmt=new TextFormat();
			fmt.size=12; 
			fmt.font = "Arial";
			fmt.bold=false;	
			body_mc.addChild(tf);
			tf.x=3;
			tf.y=3;
			pTF=tf;
			pFmt=fmt;
			pWin.x=pX;
			pWin.y=pY;
			return true ;
		}
		static private function createEvent(){
			pClose.addEventListener(MouseEvent.MOUSE_UP,closeWin);
			pHead.addEventListener(MouseEvent.MOUSE_DOWN,fDown);
			pHead.addEventListener(MouseEvent.MOUSE_UP,fUp);
		}
		
		static private function closeWin(e:Event){
			pClose.removeEventListener(MouseEvent.MOUSE_UP,closeWin);
			pHead.removeEventListener(MouseEvent.MOUSE_DOWN,fDown);
			pHead.removeEventListener(MouseEvent.MOUSE_UP,fUp);
			//
			pCtnr.removeChild(pWin);
			//
			pWin=null;
		}
		static private function fDown(e:Event){
			pWin.startDrag();
		}
		static private function fUp(e:Event){
			pWin.stopDrag();
		}
		
		
		
	}
}