﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.common { 
	import net.flash_line.as3.util.ui.ErrorFrame ;
	import net.flash_line.as3.util.global.* ;	
	import net.flash_line.as3.util.common.AlertWin;
	/**
	* ArrayX extends Array
	* <p>Used to create stack or structured Array of objects with "id" property</p>
	*/ 
	public dynamic class ArrayX extends Array {
	
		 /**
		 * @private 
		 * the classname of instance 
		 */		 
		var className:String ;			
			
		/**
		 *  Constructor 
		 */				
		public function ArrayX (...rest) {
			splice.apply(this, [0, 0].concat(rest));
		}
		/**
		* The last element index - this[length-1]
		* @return this[length-1]
		*/
		public function get last () :Object {
			return this[length-1];
		}
		/**
		* The first element index - this[0]
		* @return this[0]
		*/
		public  function get first () : Object {
			return this[0];
		}
		//
		/**
		* If Array contains objects with a String property named "id" then return the Object with the looking for id value
		* @param idVal 	id's value we are looking for.
		* @return the first object of Array which has its property "id" equal to idVal
		*/
		public function returnObjectOfId (idVal:String) :Object {
			var len:Number=length;
			for (var i=0;i<len;i++) {
				var o:Object=this[i];
				if (idVal==this[i].id) {
					return this[i] ;
				}
			}
			return null;
		}
		/**
		* If Array contains objects with a String property named "name" then return the Object with the looking for name value
		* @param nameVal 	name's value we are looking for.
		* @return the first object of Array which has its property "name" equal to nameVal
		*/
		public function returnObjectOfName (nameVal:String) :Object {
			var len:Number=length;
			for (var i=0;i<len;i++) {
				var o:Object=this[i];
				if (nameVal==this[i].name) {
					return this[i] ;
				}
			}
			return null;
		}
		/**
		* If Array contains objects with a String property named "id" then return the index in Array of the Object with the looking for id value
		* @param idVal 	id's value we are looking for.
		* @return   index position of the first object of Array which has its property "id" equal to idVal
		*/		
		public function returnIndexObjectOfId (idVal:String) :Number {
			var len:Number=length;
			for (var i=0;i<len;i++) {
				if (idVal==this[i].id) {
					return i ;
				}
			}
			return -1;
		}
		/**
		* copy inputArray in current ArrayX
		* @param idVal 	id's value we are looking for.
		* @return   index position of the first object of Array which has its property "id" equal to idVal
		*/		
		public function copyInto (inputArray:Array) : ArrayX {
			var len:Number=inputArray.length;
			for (var i=0;i<len;i++) {
				this[i]=inputArray[i];
			}
			return this;
		}
		
		/**
		* Push instance of this Class in the static public Instance class
		* @param	cn Classname
		* @param	inst Instance
		*/		
		public function setClassInfo(cn:String,inst:Object) {	
			className=cn;
			Instance.push(inst);
		}
		/**
		* if ArrayX is used as stack : 
		* <p>Clear this and launch remove() method of each element (i.e. an ArrayX of BasicTween instances)</p>
		* <p>Example if arr is an instancied ArrayX:<br/><br/>
		* arr.push(new BasicTween(...)) ; // several time<br/><br/>
		* // later...<br/>
		* arr.clear() // execute arr[i].remove() ~ BasicTween.remove() 
		* </p>
		* @param	func Method's name to be launched instead of remove()
		*/		
		public function clear(func:String=null) {	
			var t:Object;
			while ( t=pop() ) {
				if (func==null) t.remove(); 
				else t[func]();
			}			
		}
		/**
		* Alias of this.clear() - Call clear() without parameter
		* @see #clear()
		*/
		public function remove() {
			clear();
		}
		/**
		* display alert messages in one unique alert window
		* @param msg message string
		* @see net.flash_line.as3.util.common.AlertWin
		*/		
		public function alert (msg:String) {
			if (msg!=null) AlertWin.display(msg);
		}
		/**
		* display an error message
		* @param msg message string
		*/
		public function error (msg:String) {
			if (msg!=null) {
				var ef:ErrorFrame=new ErrorFrame (Var.parent,msg,300,200) ;
			}
		}		
	}
}


