﻿/**
* @author Jean-Michel Delettre (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.common {	
	import flash.display.* ;
	import flash.filters.* ;
	import flash.text.* ;
	import flash.geom.*;
	import fl.motion.*;
	import flash.display.DisplayObjectContainer ;
	import net.flash_line.as3.util.event.EventX ;
	import net.flash_line.as3.util.text.*;
	import net.flash_line.as3.util.common.AlertWin;
	import net.flash_line.as3.util.ui.ErrorFrame ;
	import net.flash_line.as3.util.global.* ;
	import net.flash_line.as3.util.io.LoaderSprite ;
	/**
	* Extends Sprite to add it several easier to use methods
	*/
	public class SpriteX extends Sprite { 
		/**
		 * @private 
		 * if used pParent is the root parent sprite of the application.
		 */
		protected var pParent			 : SpriteX		 ;
		/**
		* @private 
		 *  the parent container 
		 */		
		 protected var pCtnr			: DisplayObjectContainer		; 	
		/**
		 * @private 
		 * the classname of instance 
		 */		 
		 protected var pClassName:String ;
		/**
		 * @private 
		 * used to store last listener
		 */
		 protected var pLstFunction			 : Function		 ;
		/**
		 * @private  
		 * standard stack
		 */
		 protected var pStack			 	: ArrayX		 ;		
		 /**
		 * @private  
		 * TextField used if this sprite is a button - used by ButtonMng
		 */
		
		 private var pBmpStack				:ArrayX;
		/**
		* Constructor
		* @param	ctnr	Container where this must be added or null
		* @param	vx		x pos
		* @param	vy		y pos
		* @param	a		alpha
		* @param	n		name
		*/				
		public function SpriteX (ctnr:DisplayObjectContainer=null,vx:Number=0,vy:Number=0,a:Number=1,n:String=null) {
			super();	
			if (ctnr!=null) {
				pCtnr=ctnr;
				addInTo(pCtnr,n,vx,vy);		
			}
			alpha=a;
			pStack=new ArrayX();
			pBmpStack=new ArrayX();
		}
		//get/set
		/**
		* The standard stack
		* @return 
		*/		
		public function get stack (): ArrayX {
			return pStack;
		}
		/**
		* A standard stack
		* @param s an ArrayX
		* @see #ArrayX
		*/	
		public function set stack (s: ArrayX ){
			pStack=s;
		}
		/**
		* bmpx stack
		* @return 
		*/		
		public function get bmpStack (): ArrayX {
			return pBmpStack;
		}
		public function set bmpStack (arr: ArrayX) {
			pBmpStack=arr;
		}
		/**
		* The classname
		* @return 
		*/		
		public function get className (): String {
			return pClassName;
		}
		/**
		* The classname
		*/		
		public function set className (s: String ){
			pClassName=s;
		}
		/**
		* Receveid flashVars Object if any
		* @return 
		*/		
		public function get receivedVar (): Object {
			return this.loaderInfo.parameters;
		}
		/**
		* the root parent of the application if any or null
		* @return the root parent of the application
		*/		
		public function get rootParent (): SpriteX {
			return pParent;
		}
		/**
		* the root sprite of the application if any or null
		* @param sp the root parent of the application
		*/		
		public function set rootParent (sp:SpriteX) {
			pParent=sp;
		}
		/**
		* The logical parent container of this or null
		* @return the container parent of this
		*/		
		public function get ctnr (): DisplayObjectContainer {
			return pCtnr;
		}
		/**
		* The logical parent container of this or null
		* @param doc the container parent of this
		*/		
		public function set ctnr (doc:DisplayObjectContainer) {
			pCtnr=doc;
		}
		/**
		* A short name for parent.getChildIndex (this) or -1 if no valid this.parent
		* @return 
		*/
		public function get depth (): Number {
			if (parent!=null) {
				return parent.getChildIndex (this) ;
			}
			return -1;
		}
		public function getNewBmpx (vx:*=null,vy:*=null,w:*=null,h:*=null): Object {						
			var bmd:BitmapData = new BitmapData(width,height,true);
			bmd.draw(this);
			var bm:Bitmap=new Bitmap(bmd);
			bmpStack.push(bm);
			if (vx==null) var posx:Number=bm.x;
			else posx=Number(vx);
			if (vy==null) var posy:Number=bm.y;
			else posy=Number(vy);	
			if (w==null) var vw:Number=bm.width;
			else vw=Number(w);
			if (h==null) var vh:Number=bm.height;
			else vh=Number(h);
			var rect:Rectangle = new Rectangle(posx,posy,vw,vh);
			return {bitmapData:bmd,bitmap:bm,rectangle:rect};
		}
		public function removeBmpx (bm:Bitmap,doRemove:Boolean=false) {						
			if (bm!=null) {
				if (bm.bitmapData!=null) bm.bitmapData.dispose();	
				if (doRemove) {
					if (bm.parent!=null) bm.parent.removeChild(bm);
				}
			}
		}
		public function removeAllBmpx (doRemove:Boolean=false) {						
			var bm:Bitmap;
			while ( bm=bmpStack.pop() ) {
				removeBmpx (bm,doRemove);
			}	
		}
		/**
		* add this in a parent - give name , pos x and y
		* @param	ctnr 	parent
		* @param	n		name
		* @param	vx		x
		* @param	vy		y
		*/		
		public function addInTo(ctnr:DisplayObjectContainer=null,n:String=null,vx:Number=0,vy:Number=0) {
			if (ctnr==null) ctnr=pParent as DisplayObjectContainer;
			if (ctnr!=null) ctnr.addChild(this); else return ;
			if (n!=null) name=n;
			x=vx;
			y=vy;
		}
		/**
		* add child in this - give name , pos x and y
		* @param	ctnr 	parent
		* @param	n		name
		* @param	vx		x
		* @param	vy		y
		*/		
		public function addChildX(sp:DisplayObject=null,n:String=null,vx:Number=0,vy:Number=0,w:*=null,h:*=null)  {
			if (sp!=null) addChild(sp);
			if (n!=null) sp.name=n;	
			sp.x=vx;
			sp.y=vy;
			if (w!=null) sp.width=w;if (h!=null) sp.height=h;
		}		
		/**
		* Short method name to get a this.getChildByName(str)
		* @param	str DisplayObject.name value
		* @return a SpriteX son of this
		*/		
		public function child (str:String) : DisplayObject  {
			return getChildByName(str) ;
		}
		/**
		* Short method name to get a SpriteX(this.getChildByName(str))
		* @param	str SpriteX.name value
		* @return a SpriteX son of this
		*/
		public function childX (str:String) : SpriteX  {
			return SpriteX(getChildByName(str));
		}
		/**
		* Get a Sprite instance - son of this
		* <p>alias of childX</p>
		* @param	str The SpriteX.name value
		* @return   The SpriteX instance 
		*/
		public function getSp (str:String) : Sprite  {
			return Sprite(child (str));
		}
		/**
		* Get a SpriteX instance - son of this
		* <p>alias of childX</p>
		* @param	str The SpriteX.name value
		* @return   The SpriteX instance 
		*/
		public function getSpx (str:String) : SpriteX  {
			return childX (str);
		}
		/**
		* Get a TextField instance - son of this
		* @param	str The TextField.name value
		* @return   The TextField instance 
		*/		
		public function getTf(str:String) :TextField  {
			return TextField(child(str)) ;
		}
		/**
		* Get a TextFieldX instance - son of this
		* @param	str The TextFieldX.name value
		* @return   The TextFieldX instance 
		*/		
		public function getTfx(str:String) :TextFieldX  {
			return TextFieldX(child(str)) ;
		}
		
		
		/**
		* Get a TextInput instance - son of this
		* @private
		* @param	str A TextInput.name value
		* @return   The TextInput instance 
		*/
		
		/**
		* Get a TextArea instance - son of this
		* @private
		* @param	str A TextArea.name value
		* @return   The TextArea instance 
		*/
		
		public function getTix(str:String) :* {
			//alert("WARNING WARNING WARNING put again getTix and getTax as below");
			return child(str) ;
		}
		public function getTax(str:String) :* {
			//alert("WARNING WARNING WARNING put again getTix and getTax as below");
			return child(str) ;
		}	
		
		/* 		ici here*/
		/*
		public function getTix(str:String) :TextInputX  {
			return TextInputX(child(str)) ;
		}
		
		public function getTax(str:String) :TextAreaX  {
			return TextAreaX(child(str)) ;
		}
		*/
		
		
		/**
		* Swap depth between this and spx
		* @param	spx a display object which has same parent than this
		* @return	true if swap is valid
		*/
		public function swapDepthX (spx:DisplayObject):Boolean {
			if (parent!=null) {
				if (parent==spx.parent) {
					parent.swapChildren (this,spx) ;
					return true;
				}
			}
			return false;
		}
		/**
		* alias of swapDepthX
		* Swap depth between this and spx
		* @param	spx a display object which has same parent than this
		* @return	true if swap is valid
		*/
		public function swapDepth (spx:DisplayObject):Boolean {
			if (parent!=null) {
				if (parent==spx.parent) {
					parent.swapChildren (this,spx) ;
					return true;
				}
			}
			return false;
		}
		/**
		* Modifie display list index of this in its parent container
		* @param	n a vaild depth
		* @return 	true if operation is valid
		*/
		public function setDepthAt (n:Number):Boolean {
			if (parent!=null) {
				if (parent.numChildren>=n) {
					parent.swapChildren (this,parent.getChildAt(n)) ;
					return true ;
				}
			}
			return false;
		}
		/**
		* Remove all graphics objects in this
		*/
		public function clear () {
			graphics.clear();
		}
		/**
		* Used to inform if this.sprite is a child of a container or a child of rootParent 
		* @param	c A container 
		* @return true or false
		* @see #rootParent
		*/
		public function isChildOf (c:DisplayObjectContainer=null): Boolean {
			if (c==null && pParent!=null) c=pParent as DisplayObjectContainer;			
			return (c.contains(this));
		}
		/**
		* write a simple text in this
		* @param	str text
		* @param	vx	x
		* @param	vy  y
		* @param	sc	scale
		* @return	created textfield
		*/		
		public function write (str:String,vx:Number=0,vy:Number=0,sc:Number=1) :TextFieldX {		
			var font:String=Var.font;
			var size:Number=Var.size;
			if (font==null) font="Verdana";
			if (isNaN(size)) size=12;
			var tfx:TextFieldX=new TextFieldX(this,vx,vy,30,100,0x0,font,size,true,false);
			tfx.setText(str);
			tfx.multiline=true;tfx.autoSize ="left"; 
			tfx.scaleX=sc;tfx.scaleY=sc;
			return tfx ;				
		}
		/**
		* basic color setting
		* @param	col	Color rvb
		* @param	a	Alpha
		* @see #restoreColor()
		*/	
		public function setOneColor(col:Number=0,a:Number=1) {		
			var r=Math.floor(col/(256*256));
			var g=Math.floor((col%(256*256))/256);
			var b=(col%256);
			transform.colorTransform = new ColorTransform( 0, 0, 0, 1 ,r, g, b, 0);
			alpha=a;

		}
		/**
		* Restore original color after using setOneColor
		* @see #setOneColor()
		*/
		public function restoreColor() {	
			transform.colorTransform = new ColorTransform( 1,1,1,1,0,0,0,0);
			alpha=1;
		}
		/**
		* basic tint setting
		* @param	coef	tint coef
		* @param	col		Color rvb
		* @param	a		Alpha
		* @see #restoreTint()
		*/	
		public function setOneTint(col:Number=0,coef:Number=1,a:Number=1) {		
			var colObj = new Color( );
			colObj.setTint(col,coef);
			transform.colorTransform =colObj;			
			alpha=a;

		}		
		/**
		* Restore original tint after using setOneTint
		* @see #setOneTint()
		*/
		public function restoreTint() {		
			var colObj = new Color( );
			colObj.setTint(0,0);
			transform.colorTransform =colObj;			
			alpha=1;
		}
		public function changeSaturation (n:Number=.5) {	
			this.cacheAsBitmap = true;
			var matrix:Array = new Array();
			 matrix = matrix.concat([n, 0, 0, 0, 0]); // red
			 matrix = matrix.concat([0, n, 0, 0, 0]); // green
			 matrix = matrix.concat([0, 0,n, 0, 0]); // blue
			 matrix = matrix.concat([0, 0, 0, 1, 0]); // alpha
			 var filter:ColorMatrixFilter = new ColorMatrixFilter(matrix); 
			 this.filters = new Array(filter);
		}
		public function removeAllFilter() {
			 this.filters=new Array() ;
		}
		/**
		* override Sprite.toString() to have more details
		* @return override de toString()
		*/		
		public override function toString () :String {
			var str:String;
			if (parent!=null) {
				str = "\nname="+name +"\nparent.name="+parent.name+"\nvisible="+visible + "\nalpha="+alpha ;
				str += "\nx=" + x + " y=" + y + "\nw=" + width + " h=" + height + "\ndepth=" + parent.getChildIndex(this);
			} else {
				str = "\nname="+name +"\nvisible="+visible + "\nalpha="+alpha ;
				str += "\nx=" + x + " y=" + y + "\nw=" + width + " h=" + height ;
			}
			return str;
		}
		/**
		* Remove this from its parent
		*/
		public function remove () :String {
			if (parent != null) {
				if (parent.contains(this)) parent.removeChild(this);
			}
			clear ();
			removeAllBmpx(true);
			removeAllChild();			
			return "";
		}
		/**
		* remove all children of this
		*/		
		public function removeAllFrom() {
			for (var i:Number=0;i<numChildren;i++) {
				removeChildAt(i);
			}
		}
		/**
		* Alias of this.removeAllFrom()
		*/		
		public function removeAllChild() {
			removeAllFrom() ;
		}
		/**
		* remove a supposed child of this - no error even if child don't exist
		* @param	sp The supposed child
		* @return null 
		* <p>call usage may be : myProp=removeChild(myProp)</p>
		*/
		public function removeChildX(sp:DisplayObject)  {
			if (sp!=null) {
				removeChild(sp);
			}			
			return null;
		}
		/**
		* remove a supposed child of this using its name - no error even if childname isn't valid
		* @param	sp The supposed child
		* @return null if remove action is okay; else nothing
		* <p>call usage may be : myProp=removeChildByName(myProp.name)</p>
		*/
		public function removeChildByName(str:String) :* {
			if (str=="") return ;
			var sp:DisplayObject=child(str);
			if (sp!=null) {
				removeChild(sp);
			}			
			return null;
		}
		/**
		* Hide all children
		*/
		public function hideAllChild() {
			for (var i:Number=0;i<numChildren;i++) {
				getChildAt(i).visible=false;
			}
		}
		/**
		* hide child(str) - no error even child(str) don't exist
		* @see #child()
		* @see #showChild()
		* @param	str a Sprite name son of this
		*/
		public function hideChild(str:String="") {
			if (child(str)!=null) {
				child(str).visible=false;
			}
		}
		/**
		* show child(str) - no error even child(str) don't exist
		* @see #child()
		* @see #hideChild()
		* @param	str a Sprite name son of this
		*/
		public function showChild(str:String="") {
			if (child(str)!=null) {
				child(str).visible=true;
			}
		}
		/**
		* Show all children in this
		*/
		public function showAllChild() {
			for (var i:Number=0;i<numChildren;i++) {
				getChildAt(i).visible=true;
			}
		}
		
		/**
		* Push instance of this Class in the static public Instance class 
		* @param	cn Classname
		* @param	inst Instance
		*/		
		public function setClassInfo(cn:String,inst:Object) {	
			className=cn;
			Instance.push(inst);
		}
		/**
		* dispatchEvt() dispatch events with a net.flash_line.as3.util.event.EventX parameter
		* <p>EventX has a property srcVar which may contains diverse dynamic properties to transfert to the listeners</p>		
		* <p>All net.flash_line.as3 events have the same type which is EventX</p>
		* <p>EventX extends flash.events.Event and all event source extends flash.events.EventDispatcher</p>
		* <p>dispatchEvt() use EventDispatcher.dispatchEvent(new EventX(evtName))</p>
		* <p>It's of course possible to do not use dispathEvt() for new source event classes</p>
		* <p>but using dispatchEvt() is less heavy than original AS3 system.</p>
		* <p>When using dispatchEvt() it isn't necessary to add event name constant</p>
		* <p>in EventX except when one want to have nice comments in asdoc ;)</p>
		* @see net.flash_line.as3.util.event.EventX#srcVar		
		* @see net.flash_line.as3.util.event.EventX		
		* @param	evtName A string which is event name
		* @param	o Object with properties -parameters- which will be assigned to EventX.srcVar 
		*/
		public function dispatchEvt (evtName:String,o:Object=null) {					
			var ex:EventX=new EventX(evtName);
			ex.srcVar=o;
			dispatchEvent(ex);
		}
		/**
		* display debug messages in a unique alert win
		* @param msg message string
		* @see net.flash_line.as3.util.common.AlertWin
		*/		
		public function alert (msg:String) {
			// by default debug will be equal to true.
			var debug:Boolean = boolVal(getGlobalVar().debug, true);
			if (debug) {
				if (msg != null) AlertWin.display(msg);
			}
		}
		/**
		* display an error message
		* @param msg message string
		*/		
		
		public function showError (msg:String) {
			error(msg);			
		}
		/**
		* display in alert frame , children tree
		* <p>used only during debug time</p>
		* <p>listChild is a recursive method</p>
		* @param	doc The container - must be null, used because recursive func
		* @param	tab indent string - must absent, used because recursive func
		*/
		public function listChild(doc:DisplayObjectContainer=null,tab:String = "")	{
			if (doc==null) doc=this;
			var child:DisplayObject;
			for (var i:uint=0; i<doc.numChildren; i++) {
				child = doc.getChildAt(i);
				alert(tab+child.name+"{"); 				
				if (doc.getChildAt(i) is DisplayObjectContainer){
					listChild(DisplayObjectContainer(child), tab+"   ") ;
				}
				alert(tab+"}"); 
			}
		}
		/**
		* Create new spriteX instance from the library
		* <p><span style="color:#ff0000">deprecated</span> we use now Embed meta-tag or better -> dynamic load as often as possible !</p>
		* 
		* 
		* @param	cl	the clip Class
		* @param	vx	x pos
		* @param	vy	y pos
		* @param	id	name of clip
		* @return	a new instance of SpriteX
		*/
		public function attachNewSprite(cl:String,vx:Number=0,vy:Number=0,id:String=null) :SpriteX {						
			var ls:LoaderSprite=LoaderSprite(this);
			var library:Object = ls.loader.content;	
			var spx:SpriteX = new library[cl]();			
			addChildX(spx,id,vx,vy);
			return spx;
		}
		/**
		* Create new MovieClip instance from the library
		* <p ><span style="color:#ff0000">deprecated</span> we use now Embed meta-tag or better -> dynamic load as often as possible !</p>
		* @param	cl	the clip Class
		* @param	vx	x pos
		* @param	vy	y pos
		* @param	id	name of clip
		* @return	a new instance of MovieClip
		*/
		public function attachNewMc(cl:String,vx:Number=0,vy:Number=0,id:String=null) :MovieClip {						
			var ls:LoaderSprite=LoaderSprite(this);
			var library:Object = ls.loader.content;	
			var spx:MovieClip = new library[cl]();			
			addChildX(spx,id,vx,vy);
			return spx;
		}
		/**
		* @private
		* Not used except one time to try something !!
		* @param	evt
		* @param	f
		*/
		public function addEventListenerX(evt:String,f:Function)  {						
			pLstFunction=f;
			addEventListener(evt,f);
		}
		/**
		* @private
		* Not used except one time to try something !!
		* @param	evt eventname
		* @param	f	listener func
		*/
		public function removeEventListenerX(evt:String,f:Function=null)  {						
			if (f==null) f=pLstFunction;
			if (f!=null) {
				removeEventListener(evt,f);
				pLstFunction=null;
			} else {
				error("listener is null in removeEventListener(evt,listener) !!");
			}
			
		}
		/**
		* @private
		* used to save last listener added to removeEventListener after. 
		* @param	evt eventname
		* @param	f	listener func
		* @see		this.removeEventListener
		*/
		public override function addEventListener(evt:String,f:Function,useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false) : void  {						
			pLstFunction=f;
			super.addEventListener(evt,f,useCapture, priority, useWeakReference);
		}
		/**
		* @private
		* used to save last listener added to removeEventListener after.
		* @param	evt eventname
		* @param	f	listener func
		* @see this.addEventListener
		*/
		public override function removeEventListener(evt:String,f:Function,useCapture:Boolean = false) : void  {						
			if (f==null) {
				f=pLstFunction;
			}
			if (f!=null) {
				super.removeEventListener(evt,f,useCapture);
				pLstFunction=null;
			} else {
				error("listener is null in removeEventListener(evt,listener) !!");
			}			
		}
		
		/**
		* create textfield and set label - used if this sprite is a button managed by ButtonMng
		* @param	clickSpx	invisible click sprite of button
		* @param	lb			label of button
		* @param	tc			textColor
		* @param	font		font family
		* @param	size=null	font size
		* @param	bold		Bold true/false
		*/
		public function setButtonLabel(clickSpx:SpriteX,lb:String,tc:Number=0x0,font:String=null,size=null,bold:Boolean=true) {			
			font= getFont(font);
			size= getFontSize(size);
			var tfx:TextFieldX	=	new	TextFieldX(this,0,1,width,height-2,tc,font,size,bold,true,null,"buttonLabel");	
			tfx.setText(lb,{align:"center"} ) ;
			swapChildren(tfx, clickSpx);
		}
	}
	
}