﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.event {
	import net.flash_line.as3.util.global.* ;
	import flash.events.* ;
	import flash.utils.* ;
	import net.flash_line.as3.util.common.* ;	
	import net.flash_line.as3.util.event.* ;	
	import net.flash_line.as3.util.ui.ErrorFrame ;
	/**
	* A Clock instance call evenly a received Function. 
	* <p>Use Clock is easier than directly use Timer
	* <p>Usage : <br/>
	* var clk:Clock;<br/>
	* ...<br/>
	* clk=new Clock(myFunction,.05); // Start a clock on myFunction with period=0.05 second<br/>
	* ...<br/>
	* clk.remove(); // remove the clock so myFunction as listener<br/>
	* </p>
	*/ 
	public class Clock extends Timer  {	
		/**
		* @private
		 *  period - 1/frequency 
		 */
		 protected var pPeriod : Number; 
		/**
		* @private
		 * listener
		 */
		 protected var pF:Function; 
		 /**
		 *  the classname of instance * 
		 * @private 
		 */		 
		var pClassName:String ;
		
		//
		/**
		* constructor
		* @param	f 		listener function
		* @param	per		period - default is a frequency of 1/25 second 
		*/
		public function Clock (f:Function,per:Number=.04) {			
			super(per*1000,0);
			pPeriod=per*1000;
			pF=f;
			setClassInfo("Clock",this);
			begin();
		}
		/**
		* The classname
		*/		
		public function get className (): String {
			return pClassName;
		}
		/**
		* The classname
		*/			
		public function set className (s: String ){
			pClassName=s;
		}
		/**
		* re-start timer
		* @see #remove()
		*/
		public function begin () {	
		   addEventListener(TimerEvent.TIMER, pF);
           start();
        }
		/**
		* stop timer and remove listeners
		* @see #begin()
		* @return null
		*/
		public function remove () : * {				
			stop();
			removeEventListener(TimerEvent.TIMER, pF);
			return null;
		}
		/**
		* Push instance of this Class in the static public Instance class
		* @param	cn Classname
		* @param	inst Instance
		*/		
		public function setClassInfo(cn:String,inst:Object) {	
			className=cn;
			Instance.push(inst);
		}			
		/**
		* display alert messages in one unique alert window
		* @param msg message string
		* @see net.flash_line.as3.util.common.AlertWin
		*/		
		public function alert (msg:String) {
			if (msg!=null) AlertWin.display(msg);
		}
	}
}
