﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.event {	
	import net.flash_line.as3.util.global.* ;
	/**
	* Delegate is used to pass a list of arguments from the Function which contains the addListener instruction to the listener Function itself.
	* <p>The Delegate.pass() static method do that.<br/><br/>
	* Delegate works with event AS3 system.<br/></p>
	* <p>Usage<br/>
	* anEventDispatcher.addEventListener("eventName",Delegate.pass(myListener,arg1,arg2,arg3));<br/>
	* ...<br/>
	* private function myListener(evtObj:Event,arg1:String,arg2:Number,arg3:Object) {...}<br/>
	* ...<br/>
	* // or if we want to use removeEventListener : <br/>
	* var lst:Function; // to save the delegated listener<br/>
	* ...<br/>
	* lst=Delegate.pass(myListener,arg1,arg2,arg3) ;<br/>
	* anEventDispatcher.addEventListener("eventName",lst);<br/>
	* ...<br/>
	* private function myListener(evtObj:Event,arg1:String,arg2:Number,arg3:Object) {<br/>
	* 	anEventDispatcher.removeEventListener("eventName",lst);<br/>
	* 	...<br/>
	* }<br/>
	*/	
	public class Delegate  { 		
		/**
		* @private
		*/
		static private var _listenersObject:Object= {} ;
		/**
		* Used as second argument of EventDispatcher.addEventListener()
		* instead of the listener function
		* @param	f The listener function
		* @param	... arg A list of arguments separate by coma
		* @return 	A function which will call the listener function  f and will pass arguments to it.
		*/
		static public function pass (f:Function,... arg):Function {				
			return function(...argf) {return f.apply(this, [].concat(argf,arg));};
		}
		/**
		* @private
		* This method is not finished
		* @param	id
		* @param	src
		* @param	evt
		* @param	lst
		* @return
		*/
		static public function add (id:String,src:*,evt:String,lst:Function):Boolean {				
			if (_listenersObject[id]!=null) {
				error("In Delegate.add() "+id+" already exist");
				return false;
			}
			_listenersObject[id]={};
			_listenersObject[id].src=src;
			_listenersObject[id].evt=evt;
			_listenersObject[id].lst=lst;
			src.addEventListener(evt,lst);
			return true ;
		}		
		/**
		* @private
		* This method is not finished
		* @param	id
		*/
		static public function remove (id:String){				
			if (_listenersObject[id]==null) {				
				return false;
			}
			var o:Object=_listenersObject[id];
			o.src.removeEventListener(o.evt,o.lst);
			suppress(id);
		}
		/**
		* @private
		* This method is not finished
		*/
		static public function suppress (id:String){				
			if (_listenersObject[id]==null) {
				error("In Delegate.supress() : "+id+" don't exist");
				return false;
			}
			_listenersObject[id]=null;
		}
		
	}
}

