﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.event {
	import flash.events.* ;
	import net.flash_line.as3.util.global.* ;
	/**
	* Contains all specific Event objects of net.flash_line.as3 classes.
	* <p>It has only one property : srcVar</p>
	* @see #srcVar
	* @see #ON_LOAD_PROGRESS
	*/
	public class EventX extends Event { 
		/**
		* Object with event source vars to transfert to listeners
		* @private
		*/
		protected var pSrcVar:Object ;
		
		/**
		 * The constant EventX.VIDEO_INIT define the value of the
		 * property <code>type</code> of a <code>videoInit</code> event.
		 *
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.errMsg</td><td>  A string equal to "noValidSrc" , "streamErr" , "connectErr" , "timeoutErr" ... or "" if okay</td></tr>
		 * </table>		
		 * @eventType videoInit
		 */
		public static const VIDEO_INIT : String ="videoInit" ;
		//
		/**
		 * The constant EventX.ON_PARSE_XML define the value of the
		 * property <code>type</code> of a <code>onParseXml</code> event.
		 *
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.tree</td><td>Object of Object or Array of...properties relating to the xml structure<p>tree start from the inner of the first node </p></td></tr>
		 * </table>		
		 * @eventType onParseXml
		 * @see net.flash_line.as3.util.xml.XmlParser 
		 */
		public static const ON_PARSE_XML : String ="onParseXml" ;		
		//
		/**
		 * The constant EventX.ON_LOAD_SKIN define the value of the
		 * property <code>type</code> of a <code>onLoadSkin</code> event.
		 *
		 * <p>The event object property srcVar has no property :</p>
		 * @eventType onLoadSkin
		 * @see net.flash_line.as3.util.io.SkinLoader
		 */
		public static const ON_LOAD_SKIN : String ="onLoadSkin" ;
		//
		/**
		 * The constant EventX.ON_LOAD_IMAGE define the value of the
		 * property <code>type</code> of a <code>onLoadImage</code> event.
		 *
		 * <p>The event object prperty srcVar has no property :</p>
		 * @eventType onLoadImage
		 */
		public static const ON_LOAD_IMAGE : String ="onLoadImage" ;
		//
		/**
		 * The constant EventX.ON_LOAD_CONNECTING define the value of the
		 * property <code>type</code> of a <code>onLoadConnecting</code> event.
		 *
		 * <p>The event object property srcVar has no property :</p>
		 * @eventType onLoadConnecting
		 */
		public static const ON_LOAD_CONNECTING : String ="onLoadConnecting" ;
		//
		/**
		 * The constant EventX.ON_LOAD_MODEL define the value of the
		 * property <code>type</code> of a <code>onLoadModel</code> event.
		 *
		 * <p>The event object property srcVar has no property :</p>
		 * @eventType onLoadModel
		 */
		public static const ON_LOAD_MODEL : String ="onLoadModel" ;
		//
		/**
		 * The constant EventX.ON_LOAD_INIT define the value of the
		 * property <code>type</code> of a <code>onLoadInit</code> event.
		 *
		 * <p>The event object property srcVar has no property :</p>
		 * @eventType onLoadInit
		 * @see net.flash_line.as3.util.io.LoaderSprite
		 */
		public static const ON_LOAD_INIT : String ="onLoadInit" ;
		//
		/**
		 * The constant EventX.ON_LOAD_COMPLETE define the value of the
		 * property <code>type</code> of a <code>onLoadComplete</code> event.
		 *
		 * <p>The event object property srcVar has no property </p>
		 * @eventType onLoadComplete
		 * @see net.flash_line.as3.util.io.LoaderSprite
		 */
		public static const ON_LOAD_COMPLETE : String ="onLoadComplete" ;
		//
		/**
		 * The constant EventX.ON_LOAD_ERROR define the value of the
		 * property <code>type</code> of a <code>onLoadError</code> event.
		 *
		 * <p>The event object property srcVar has no property </p>
		 * @eventType onLoadError
		 * @see net.flash_line.as3.util.io.LoaderSprite		 
		 */
		 public static const ON_LOAD_ERROR : String ="onLoadError" ;
		//
		/**
		 * The constant EventX.ON_LOAD_PROGRESS define the value of the
		 * property <code>type</code> of a <code>onLoadProgress</code> event.
		 *
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.total</td><td>Total bytes of media</td></tr>
		 * <tr><td>srcVar.loaded</td><td>Current loaded bytes of media</td></tr>
		 * </table>	
		 * @eventType onLoadProgress
		 * @see net.flash_line.as3.util.io.LoaderSprite		 
		 */
		public static const ON_LOAD_PROGRESS : String ="onLoadProgress" ;
		//
		//
		/**
		 * The constant EventX.ON_DELAY define the value of the
		 * property <code>type</code> of a <code>onDelay</code> event.
		 * <p>The event object property srcVar has no property :</p>
		 * @eventType onDelay
		 * @see Delay2#dispatchEvt()
		 */
		public static const ON_DELAY : String ="onDelay" ;
		//
		//
		/**
		 * The constant EventX.ON_SEND_AND_LOAD define the value of the
		 * property <code>type</code> of a <code>onSendAndLoad</code> event.
		 *
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.data</td><td>an UrlVariables object</td></tr>
		 * </table>	
		 * @eventType onSendAndLoad
		 * @see net.flash_line.as3.util.io.Xdata#sendAndLoadVars()
		 */
		public static const ON_SEND_AND_LOAD : String ="onSendAndLoad" ;
		//		
		/**
		 * The constant EventX.ON_LOAD define the value of the
		 * property <code>type</code> of a <code>onLoad</code> event.
		 *
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.data</td><td>an UrlVariables object</td></tr>
		 * </table>	
		 * @eventType onLoad
		 * @see net.flash_line.as3.util.io.Xdata#loadVars()
		 */
		public static const ON_LOAD : String ="onLoad" ;
		//
		/**
		 * The constant EventX.ON_LOAD_XML define the value of the
		 * property <code>type</code> of a <code>onLoadXml</code> event.
		 *
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.xml</td><td>an XML object</td></tr>
		 * </table>	
		 * @eventType onLoadXml
		 * @see net.flash_line.as3.util.io.Xdata#loadXml()
		 * @see net.flash_line.as3.util.io.Xdata#sendVarAndloadXml()
		 */
		public static const ON_LOAD_XML : String ="onLoadXml" ;
		//
		/**
		 * The constant EventX.ON_ERROR_LOAD_DATA define the value of the
		 * property <code>type</code> of a <code>onErrorLoadData</code> event.
		 *
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.error</td><td>the string "loadError"</td></tr>
		 * </table>	
		 * @eventType onErrorLoadData
		 * @see net.flash_line.as3.util.io.Xdata#loadVars()
		 * @see net.flash_line.as3.util.io.Xdata#sendAndLoadVars()
		 */
		public static const ON_ERROR_LOAD_DATA : String ="onErrorLoadData" ;
		//
		/**
		 * The constant EventX.ON_ERROR_LOAD_XML define the value of the
		 * property <code>type</code> of a <code>onErrorLoadXml</code> event.
		 *
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.error</td><td>the string "loadError"</td></tr>
		 * </table>	
		 * @eventType onErrorLoadXml
		 * @see net.flash_line.as3.util.io.Xdata#loadXml()
		 * @see net.flash_line.as3.util.io.Xdata#sendVarAndLoadXml()
		 */
		public static const ON_ERROR_LOAD_XML : String ="onErrorLoadXml" ;
		//
		/**
		 * The constant EventX.ON_LOAD_BATCH define the value of the
		 * property <code>type</code> of a <code>onLoadBatch</code> event.
		 *
		 * <p>The event object property srcVar has no property </p>
		 * @eventType onLoadBatch
		 * @see net.flash_line.as3.util.io.BatchLoader
		 */
		public static const ON_LOAD_BATCH : String ="onLoadBatch" ;
		//
		/**
		 * The constant EventX.ON_END_TWEEN define the value of the
		 * property <code>type</code> of a <code>onEndTween</code> event.
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.tween</td><td>BasicTween instance</td></tr>
		 * </table>	
		 * @eventType onEndTween
		 * @see net.flash_line.as3.util.motion.BasicTween
		 */
		public static const ON_END_TWEEN : String ="onEndTween" ;
		//
		/**
		 * The constant EventX.ON_CHANGE_TWEEN define the value of the
		 * property <code>type</code> of a <code>onChangeTween</code> event.
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.tween</td><td>BasicTween instance</td></tr>
		 * </table>	
		 * @eventType onChangeTween
		 * @see net.flash_line.as3.util.motion.BasicTween
		 */
		public static const ON_CHANGE_TWEEN : String ="onChangeTween" ;
		//
		/**
		 * The constant EventX.ON_DISPLAY_BUBBLE define the value of the
		 * property <code>type</code> of a <code>onDisplayBubble</code> event.
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>srcVar.target</td><td>an info-bubble sprite</td></tr>
		 * </table>	
		 * @eventType onDisplayBubble
		 * @see net.flash_line.as3.util.ui.bubble.BubbleInfo
		 */
		public static const ON_DISPLAY_BUBBLE : String ="onDisplayBubble" ;
		//
		/**
		 * The constant EventX.ON_REMOVE define the value of the
		 * property <code>type</code> of a <code>onRemove</code> event.
		 * <p>The event object property srcVar has no property :</p>
		 * @eventType onRemove
		 */
		public static const ON_REMOVE : String ="onRemove" ;
		/**
		 * The constant EventX.ON_IMAGE_DISPLAY define the value of the
		 * property <code>type</code> of a <code>onImageDisplay</code> event.
		 * <p>The event object property srcVar has no property :</p>
		 * @eventType onImageDisplay
		 */
		public static const ON_IMAGE_DISPLAY : String ="onImageDisplay" ;
		/**
		 * The constant EventX.ON_CUE_POINT define the value of the
		 * property <code>type</code> of a <code>onCuePoint</code> event.
		 * <p>The event object property is :</p>
		 * <table class=innertable>
		 * <tr><th>Property</th><th>Value</th></tr>
		 * <tr><td>cuePointValue</td><td>  the value of current cue point - in second</td></tr>
		 * <tr><td>cuePointIndex</td><td>  the order of current cue point </td></tr>
		 * </table>	
		 * @eventType onCuePoint
		 */
		public static const ON_CUE_POINT : String ="onCuePoint" ;
		public var cuePointValue:Number;
		public var cuePointIndex:Number;
		
		
		
		
		//
		//
		/**
		* constructor
		* @param	s event's name
		*/	
		public function EventX (s:String) {
			super(s);
		}
		/**
		* Object with event source vars to transfert to listeners
		* <p>EventX.srcVar is common for all net.flash_line.as3 event source methods</p>
		* @see net.flash_line.as3.util.common.SpriteX#dispatchEvt()
		* @see net.flash_line.as3.util.common.Super#dispatchEvt()
		* @param o Object with event source vars to transfert to listeners
		*/
		public function set srcVar (o:Object) : void {
			pSrcVar=o;
		}
		/**
		* Object with event source vars to transfert to listeners
		* <p>EventX.srcVar is common for all net.flash_line.as3 event source methods</p>
		* @see net.flash_line.as3.util.common.SpriteX#dispatchEvt()
		* @see net.flash_line.as3.util.common.Super#dispatchEvt()
		* @return Object with event source vars to transfert to listeners
		*/
		public function get srcVar ():Object {
			return pSrcVar;
		}
		
		
				
		
		
	}
}