﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.event {
	import flash.events.* ;
	import flash.display.* ;
	import flash.utils.* ;
	import net.flash_line.as3.util.common.* ;	
	import net.flash_line.as3.util.global.* ;	
	import net.flash_line.as3.util.event.* ;	
	import net.flash_line.as3.util.ui.ErrorFrame ;
	/**
     * Dispatched at the end of given period.
	 * @eventType net.flash_line.as3.util.event.EventX.ON_DELAY
	 * @see #dispatchEvt()
     */
    [Event( name="onDelay", type="net.flash_line.as3.util.event.EventX" )]
	/**
	* Send an "onDelay" event after n seconds | n is decimal.
	* Pause extends and is similar to Timer.
	* <p>
	* usage :<br/>
	* new Pause(3).addEventListener(EventX.ON_DELAY,fTimeOut); // FtimeOut Function will be called after 3 seconds<br/>
	* <br/>	
	*/ 
	public class Pause extends Timer implements ISuper {
		/**
		* @private
		 *  period - 1/frequency 
		 */
		 protected var pPeriod : Number; 
		/**
		* @private
		 * listener
		 */
		 protected var pF:Function; 
		 //
		 include "./../common/isuper.inc.as";
		/**
		* constructor
		* @param	per		period - default is a frequency of 1/25 second 
		* @param	d Delay in second before onDelay event
		*/
		public function Pause(f:Function,per:Number=1) {
			super(per*1000,1);
			pPeriod = per * 1000;	
			pF=f;
			setClassInfo("Pause",this);
			begin();			
		}
		/**
		* @private
		* start 
		*/
		public function begin () {	
		   addEventListener(TimerEvent.TIMER, pF);
           addEventListener(TimerEvent.TIMER, onEndPause);
           start();
        }
		/**
		* stop  and remove listeners
		* @return null
		*/
		public function remove () : * {				
			stop();
			removeEventListener(TimerEvent.TIMER,pF);
			return null;
		}	
		/**
		* alias of remove to test
		*/
		public function clear () : * {	
			return remove();
		}	
		
		// Private
		/**
		* listen and transmit event
		* @param	e
		*/
		private function onEndPause (e:TimerEvent) {					
			remove();
			e.updateAfterEvent();
			
		}
		
	}
}