﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.graph {		
	import flash.display.DisplayObjectContainer ;
	import flash.text.TextFormat;
	import net.flash_line.as3.util.text.TextFieldX ;
	import net.flash_line.as3.util.global.* ; // used for getFont() at least
	/**
	* Create a visible standard button.
	* <p>Graphs are dynamicly made</p>
	*/
	public class  BasicButton  extends Graph {
			
			private var pCr			: Number		;	// corner radius
			private var pMc			: RoundRect	;	// mouse interactive hidden button
			private var pOverMc		: RoundRect	;	// visible button on rollOver
			private var pOutMc		: RoundRect	;	// visible button on rollOut ( normal state)
			private var pDownMc		: RoundRect	;	// visible button on rollOut ( normal state)
			private var pTf			: TextFieldX	;
		/**
		* constructor
		* @param	ctnr	Container parent
		* @param	vx		x pos
		* @param	vy		y pos
		* @param	w		width
		* @param	h		height
		* @param	cf		fill color
		* @param	af		fill alpha
		* @param	cl		line color
		* @param	al		line alpha
		* @param 	cr		corner radius
		*/		
		public function BasicButton	  ( ctnr:DisplayObjectContainer,vx:Number=0,vy:Number=0,w:Number=100,h:Number=100,cf:Number=0xffffff,af:Number=1,cl:Number=0x0,al:Number=1,cr:Number=8) {
			super (ctnr,vx,vy,w,h,cf,af,cl,al)	;
			pCr=cr;
			createMainButton(pW,pH);
			createButton("out",cf,af,cl,al);
			setClassInfo("BasicButton",this);
		}
		//get/set
		/**
		* visible graph button when rollOver
		*/
		public function get overMc() :RoundRect  {
			return pOverMc;
		}
		/**
		* visible graph button when rollOut or normal
		*/
		public function get outMc() :RoundRect  {
			return pOutMc;
		}
		/**
		* visible graph button when down -press-
		*/
		public function get downMc() :RoundRect  {
			return pDownMc;
		}
		/**
		* main,interactive and not visible button
		*/
		public function get mc() :RoundRect  {
			return pMc;
		}		
		/**
		* TextField of button
		*/
		public function get tf() :TextFieldX  {
			return pTf;
		}
		/**
		* create interactive and not visible button
		* @param	w	width
		* @param	h	height
		*/
		public function createMainButton( w:Number=100,h:Number=100) {
			if (pMc!=null) pMc.remove();
			pMc=new RoundRect(this,0,0,w,h,0,0,0,0);			
		}
		/**
		* create an inner button "out","over" or "down"
		* @param	type 	"out","over","down"
		* @param	w		width
		* @param	h		height
		* @param	cf		fill color
		* @param	af		fill alpha
		* @param	cl		line color
		* @param	al		line alpha
		* @param	cr		corner radius
		*/
		public function createButton ( type:String,cf:Number=0xffffff,af:Number=1,cl:Number=0x0,al:Number=1) {
			var mc:RoundRect=	new RoundRect(this,0,0,pW,pH,cf,af,cl,al,pCr);
			swapChildren(mc, pMc);
			if (type=="out") {
				if (pOutMc!=null) pOutMc.remove();
				pOutMc=mc;
				pOutMc.visible=true;
			} else {
				if (type=="over") {
					if(pOverMc!=null) pOverMc.remove();
					pOverMc=mc;					
				} else if (type=="down") {
					if(pDownMc!=null) pDownMc.remove();
					pDownMc=mc;
				}
				mc.visible=false;
			}
		}
		/**
		* remove all inner buttons except main invisible and interactive one
		*/
		public function removeGraphButton ( ) {
			removeChildX(pDownMc);
			removeChildX(pOutMc);
			removeChildX(pOverMc);
		}
		/**
		* set label textfield
		* @param	lb		label
		* @param	tc		text color
		* @param	font	font name
		* @param	size	font size
		* @param	bold	true/false
		*/
		public function setLabel(lb:String,tc:Number=0x0,font:String=null,size=null,bold:Boolean=true,emb:Boolean=true,tfo:Object=null,txf:TextFormat=null) {
			if (tfo == null) tfo = { align:"center" };
			if (pTf!=null) pTf.remove();
			font= getFont(font);
			size = getFontSize(size);
			pTf	=	new	TextFieldX(this, 3, 1, pW, pH - 2, tc, font, size, bold, emb);	
			pTf.setText(lb,tfo,txf ) ;
			swapChildren(pTf, pMc);
		}
	}
}