﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.io {		
	import flash.events.* ;
	import net.flash_line.as3.util.global.* ;
	import net.flash_line.as3.util.common.* ;	
	import net.flash_line.as3.util.event.* ;
	/**
     * Dispatched when all files are loaded
	 * @eventType net.flash_line.as3.util.event.EventX.ON_LOAD_BATCH
	 * @see net.flash_line.as3.util.common.SpriteX#dispatchEvt()
     */
    [Event( name="onLoadBatch", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when an error occure during batch loading
	 * @eventType net.flash_line.as3.util.event.EventX.ON_LOAD_ERROR
	 * @see net.flash_line.as3.util.common.SpriteX#dispatchEvt()
     */
    [Event( name="onLoadError", type="net.flash_line.as3.util.event.EventX" )]
	
	/**
	* Load a batch of file and add each as child of designed parent sprite.
	*
	*/ 
	public class BatchLoader extends Super {
		private var pBase:String;//  Base pathname - common for all batch files  
		private var pClk:Clock;//  To detect time out		
		private var pErrMsg:String;// default error message if any
		private var pProgressType:String;// PROGRESS_BY_FILE or PROGRESS_BY_BYTE
		//
		/**
		*  progressByFile means load progress = number of file loaded and total = total files number.
		*/
		public static const PROGRESS_BY_FILE : String ="progressByFile" ;
		/**
		*  progressByByte means load progress = number of files bytes loaded and total = total bytes of all files.
		*/
		public static const PROGRESS_BY_BYTE : String ="progressByByte" ;
		/**
		* constructor
		*/
		public function BatchLoader (base:String="",progressType:String="progressByFile") {		
			setClassInfo("BatchLoader",this);
			stack=new ArrayX();
			pBase=base;
			pProgressType=progressType ;
		}
		//get/set
		/**
		* 
		*
		*/
		public function get fileList () : ArrayX {
			return stack;
		}		
		public function push(ctnr:SpriteX,file:String,vn:String="",vx:*=null,vy:*=null,w=null,h=null,rel:Boolean=false):Number {
			var o={ctnr:ctnr,file:file,name:vn,x:vx,y:vy,width:w,height:h,rel:rel,isLoaded:false }
			stack.push(o);
			return stack.length-1 ;  
		}
		/**
		* remove listeners and Clock 
		* @return null
		*/
		public function remove() {
			removeClock();
			removeListener () ;
			return null;
		}
		public function removeLoadedFile() {
			var stkLen:Number=stack.length;
			for (var i=0;i<stkLen;i++) {
				stack[i].sprite.remove();
			}
		}
		
		//
		//
		//
		/**
		* 
		* @param	timeOut 
		* @param	errMsg If ==null no message is displayed by BatchLoader - the caller have to display message
		*/
		public function load (timeOut:Number=10,errMsg:String=null) {
			pErrMsg=errMsg;
			var vx;
			var vy;
			var o:Object;
			var src:String;
			var stkLen:Number=stack.length;
			stack.lastLoaded=0;			
			stack.loaded=0;	
			pClk=new Clock(fOnTimeOut,timeOut);
			//			
			for (var i:Number=0;i<stkLen;i++) {				
				o=stack[i]; 				
				vx=o.x;
				vy=o.y;
				if (isStrSet(o.file)) {					
					if (o.file.substring(0,4).toLowerCase() !="http") src=pBase+o.file ;
					else src=o.file;
					o.sprite=new LoaderSprite(o.ctnr,src,vx,vy,o.width,o.height,o.rel);						
					o.loaderCompFunc=Delegate.pass(fOnLoadFile,o);
					o.loaderProgFunc=Delegate.pass(fOnLoadProgress,o)
					o.loaderErrFunc=Delegate.pass(fOnLoadError,o) ;
					o.sprite.addEventListener("onLoadComplete",o.loaderCompFunc);
					o.sprite.addEventListener("onLoadProgress",o.loaderProgFunc);
					o.sprite.addEventListener("onLoadError",o.loaderErrFunc);
					o.sprite.name=strVal(o.name);
				} else  {
					//alert("Batch load error ! Filename is "+strVal(o.name,"unknow name") );
				}
			}
		}
		private function fOnLoadProgress (e:EventX,o:Object) {			
			var loaded:Number;var total:Number;
			loaded=e.srcVar.loaded;total=e.srcVar.total;
			o.loaded=numVal(o.loaded,0) ; o.total=numVal(o.total,0) ; 
			loaded=numVal(loaded,0) ; total=numVal(total,0) ; 
			var addLoaded=loaded-o.loaded;
			o.loaded=loaded ; 	o.total= total;
			stack.loaded=numVal(stack.loaded,0) ; stack.total=numVal(stack.total,0) ; 
			if (boolVal(o.firstProgress,true) && numVal(total,0)>0) {
				stack.total+=total;
				o.firstProgress=false;
			}
			stack.loaded+=addLoaded ;
			if (pProgressType==PROGRESS_BY_BYTE) dispatchEvt(EventX.ON_LOAD_PROGRESS,{total:stack.total,loaded:stack.loaded}) ;
			else {
				loaded=0;total=stack.length;
				for (var i=0;i<total;i++) {
					if (boolVal(stack[i].isLoaded,false) ) loaded++;
				}
				dispatchEvt(EventX.ON_LOAD_PROGRESS,{total:total,loaded:loaded});
			}
		}
		/* version bis if by bytes
		private function fOnLoadProgress (e:EventX,o:Object) {
			var loaded:Number;var total:Number;
			loaded=numVal(e.srcVar.loaded,0);total=numVal(e.srcVar.total,0);
			if (total>0) {				
				o.total=total;
			}
			var totalOk=true;stack.total=0;
			for (var i=0;i<stack.length;i++) {
				if (stack[i].total<1) totalOk=false ;
				else stack.total+=stack[i].total;
			}
			if (totalOk  ) {	
				o.loaded=numVal(o.loaded,0);
				var addLoaded=loaded-o.loaded;
				o.loaded=loaded ; 
				stack.loaded=numVal(stack.loaded,0) ; stack.total=numVal(stack.total,0) ;
				stack.loaded+=addLoaded ;
				dispatchEvt(EventX.ON_LOAD_PROGRESS,{total:stack.total,loaded:stack.loaded});
			}	
		}
		*/
		private function fOnLoadError (e:EventX,o:Object) {
			//TO DO if necessary
		}
		private function fOnLoadFile (e:EventX,o:Object) {
			var isLoaded:Boolean=true;			
			o.isLoaded=true ;
			for (var i=0;i<stack.length;i++) {
				if (!boolVal(stack[i].isLoaded) ) isLoaded=false ;
			}
			if (isLoaded == true ) {				
				dispatchEvt(EventX.ON_LOAD_PROGRESS,{total:stack.total,loaded:stack.loaded});
				removeClock();	
				removeListener();
				pClk=null;	
				dispatchEvt(EventX.ON_LOAD_BATCH);	
				
			}			
		}		
		private function removeListener () {
			var stkLen:Number=stack.length;
			for (var i = 0; i < stkLen; i++) {
				if (stack[i].sprite!=null) {
					if (stack[i].sprite.hasEventListener("onLoadComplete") && stack[i].loaderCompFunc!=null ) {					
						stack[i].sprite.removeEventListener("onLoadComplete",stack[i].loaderCompFunc);
					}
					if (stack[i].sprite.hasEventListener("onLoadProgress") && stack[i].loaderProgFunc!=null ) {
						stack[i].sprite.removeEventListener("onLoadProgress",stack[i].loaderProgFunc);
					}
					if (stack[i].sprite.hasEventListener("onLoadError") && stack[i].loaderErrFunc!=null ) {
						stack[i].sprite.removeEventListener("onLoadError",stack[i].loaderErrFunc);
					}
				}
			}
		}
		private function removeClock() {
			if (pClk!=null) {
				pClk=pClk.remove();	
			}
		}
		private function fOnTimeOut (e:Event) {			
			//var arr:Array=Array(stack);
			if (stack.loaded!=stack.lastLoaded) {
				stack.lastLoaded=stack.loaded ;
			} else {
				//alert("Batch load error :");
				var stkLen:Number=stack.length;
				for (var i=0;i<stkLen;i++) {
					//alert("stack[i].file="+stack[i]);					
					if (!boolVal(stack[i].isLoaded) ) {
						//alert(" This file is not loaded="+stack[i].file);
					}
				}
				if (pErrMsg!=null) error(pErrMsg);
				dispatchEvt(EventX.ON_LOAD_ERROR);	
				removeClock();	
			}
			
		}				
		
	}//End of Class BatchLoader
}
