﻿/**
* @author Jean-Michel Delettre (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.io {	
	import flash.display.* ;
	import flash.text.* ;
	import flash.system.Security;
	import flash.system.ApplicationDomain;
	import flash.system.SecurityDomain
    import flash.system.LoaderContext;
	import flash.net.*;
	import flash.events.*;
	import net.flash_line.as3.util.common.*;
	import net.flash_line.as3.util.event.*;
	/**
     * Dispatched when media loading init is correct
	 * @eventType net.flash_line.as3.util.event.EventX.ON_LOAD_INIT
	 * @see net.flash_line.as3.util.common.SpriteX#dispatchEvt()
     */
    [Event( name="onLoadInit", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when media loading is finish.
	 * @eventType net.flash_line.as3.util.event.EventX.ON_LOAD_COMPLETE
	 * @see net.flash_line.as3.util.common.SpriteX#dispatchEvt()
     */
    [Event( name="onLoadComplete", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when an error occure during loading
	 * @eventType net.flash_line.as3.util.event.EventX.ON_LOAD_ERROR
	 * @see net.flash_line.as3.util.common.SpriteX#dispatchEvt()
     */
    [Event( name="onLoadError", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched during loading to send current loaded bytes
	 * @eventType net.flash_line.as3.util.event.EventX.ON_LOAD_PROGRESS
	 * @see net.flash_line.as3.util.common.SpriteX#dispatchEvt()
     */
    [Event( name="onLoadProgress", type="net.flash_line.as3.util.event.EventX" )]
	
	
	/**
	* Load external media (swf,png, etc) in this 
	* <p>
	* Almost same code than LoaderSprite
	* The difference with LoaderSprite is : Children of Loader.content are not embeded in a SpriteX.
	* Set its x,y,width,height ; media may be centered inside this SpriteX </p>
	* @see net.flash_line.as3.util.common.SpriteX
	*/
	public class LoaderX extends SpriteX {
		/**
		* @private
		*/
		var pLoader						:Loader;
		private var pType				:String ; //"swf" or "bmp"
		private var pFile				:String ; // filename to be loaded
		private var pRel				:Boolean ; 
		private var pW					:Number; 
		private var pH					:Number ; 
		private var pX					:Number; 
		private var pY					:Number ; 
		/**
		* constructor
		* @param	ctnr		container
		* @param	file		filename to be loaded
		* @param	vx			pos x if absolute
		* @param	vy			pos y if absolute
		* @param	w			width 
		* @param	h			height
		* @param	rel			if true : x,y,width,height are re-calculate - image is centered , reduced if needed but never bigger than original file.
		*/
		public function LoaderX (ctnr:DisplayObjectContainer,file:String,vx=null,vy=null,w=null,h=null,rel:Boolean=false) {
			super(ctnr,0,0,1);	
			pFile			= file;
			if (w!=null) pW	= Number(w)		;
			if (h!=null) pH	= Number(h)		;
			if (vx!=null) pX= Number(vx)	;
			if (vy!=null) pY= Number(vy)	;
			pRel	= rel	;
			if (w==null && h==null) pRel=false;
			if (pFile.substring(pFile.length-4,pFile.length)==".swf") pType="swf" ; else pType="bmp" ;
			load() ;
		}
		//get/set
		/**
		* Loader instance
		* <p>Even if we transfert swf content in this SpriteX we keep a trace of Loader instance to access to the swf's library's movieClips</p>
		* @return Loader instance
		* 
		*/
		public function get loader () : Loader{
			return pLoader;
		}
		/**
		* Loaded filename 
		* @return loaded filename 
		*/
		public function get file () : String {
			return pFile;
		}
		/**
		* remove Loader instance
		*/
		public function removeLoader() {
			if (pLoader!=null) {
				pLoader.unload();
				pLoader = null;
			}
		}
		/**
		* remove this
		*/
		override public function remove ():String {
			removeLoader();
			return super.remove();
		}
		/**
		* effective load
		*/
		private function load() {
			var context:LoaderContext = new LoaderContext();
			if (Security.sandboxType==Security.REMOTE) {
				if (pType=="bmp") {	
					context.checkPolicyFile = true;
				} else {
					context.securityDomain = SecurityDomain.currentDomain;
					context.applicationDomain = ApplicationDomain.currentDomain;
				}
			}
			pLoader = new Loader();
			createEvent(pLoader.contentLoaderInfo);
            pLoader.load(new URLRequest(pFile),context);
		}
		
		
		/**
		* create listeners and dispatch to caller
		* @param	ed	Native Event dispacher
		*/
		private function createEvent(ed:IEventDispatcher) {
		   ed.addEventListener(Event.COMPLETE, fLoadComplete);
           ed.addEventListener(Event.INIT, fLoadInit);
           ed.addEventListener(IOErrorEvent.IO_ERROR, fLoadError);
           ed.addEventListener(ProgressEvent.PROGRESS, fLoadProgress);
		}
		/**
		* listen Loader.contentLoaderInfo "INIT"
		* @param	evt event src Object
		* @dispatch "onLoadInit"
		*/
		private function fLoadInit(evt:Event) {
			dispatchEvt("onLoadInit");
		}
		/**
		* listen Loader.contentLoaderInfo "COMPLETE"
		* @param	evt event src Object
		* @dispatch "onLoadComplete"
		*/
		private function fLoadComplete(evt:Event) {	
			transfertInThis();
			setXYandResize();
			dispatchEvt("onLoadComplete");			
		}
		/**
		* listen Loader.contentLoaderInfo "IO_ERROR"
		* @param	evt event src Object
		* @dispatch "onLoadError"
		*/
		private function fLoadError(evt:Event) {
			showError("load error : "+pFile);
			dispatchEvt("onLoadError"); 
		}
		/**
		* listen Loader.contentLoaderInfo "PROGRESS"
		* @param	evt event src Object
		* @dispatch "onLoadProgress"
		*/
		private function fLoadProgress(evt:ProgressEvent) {			
			if (evt.bytesTotal>0 && evt.bytesLoaded>0) {
				dispatchEvt("onLoadProgress",{total:evt.bytesTotal,loaded:evt.bytesLoaded});
			}
		}
		/**
		* Transfert loaded object in this SpriteX
		*/
		private function transfertInThis() {
			if (pType=="swf" ) transfertMc();
			else transfertBmp();			
		}
		/**
		* Transfert loaded swf children in this SpriteX
		*/
		private function transfertMc() {	
			var lct:DisplayObjectContainer=pLoader.content as DisplayObjectContainer ;	
			if (lct==null) {
				transfertBmp() ;
				return;
			}
			var n:Number = lct.numChildren;
			for (var i:Number = 0; i < n; i++) {
				var sp:DisplayObject=lct.getChildAt(0) ;
				addChildX(sp,null,sp.x,sp.y); 
			}
		}
		/**
		* Transfert the Loader in this SpriteX - if bitmap 
		*/
		private function transfertBmp() {	
			var sp:DisplayObject=pLoader ;
			addChildX(sp,"img",0,0);
			//alert("inner bmp="+this.getChildAt(0).name);
		}
		/**
		* x,y,width and height settings
		*/
		private function setXYandResize() {
			if (pRel) {
				width = Math.min(pW ,width) ;
				scaleY= scaleX ;
				height = Math.min(pH,height) ;				
				scaleX= scaleY ;	
				x = (pW  - width ) / 2 ;
				y = (pH  - height ) / 2 ;
			} else {					
				if (!isNaN(pW)) width 	= Number(pW) ;
				if (!isNaN(pH)) height 	= Number(pH) ;
			}
			if (!isNaN(pX)) 	x 		= pX 		;
			if (!isNaN(pY)) 	y		= pY		;	
			
		}
	}
}
