﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.io {		
	import flash.events.* ;
	import net.flash_line.as3.util.global.* ;
	import net.flash_line.as3.util.common.Super ;
	import net.flash_line.as3.util.common.SpriteX;	
	import net.flash_line.as3.util.event.* ;
	import net.flash_line.as3.util.xml.XmlParser ;
	/**
     * Dispatched when all skin assets are loaded
	 * @eventType net.flash_line.as3.util.event.EventX.ON_LOAD_SKIN
	 * @see net.flash_line.as3.util.common.SpriteX#dispatchEvt()
     */
    [Event( name="onLoadSkin", type="net.flash_line.as3.util.event.EventX" )]
	/**
	* Load all assets of a skin defined in an xml description file.
	* <p>After the sprites will be structured in the AS3 display list as the assets are in the xml file<br/>
	* Sprites x,y,alpha,rotation,visible, etc are defined in xml file or by default</p>
	* @see http://www.flash-line.net/intra/freeFlv/xml/skin.xml
	*/ 
	public class SkinLoader extends Super {
		private var pSkin:Object;
		private var pBase:String;
		private var pFile:String;
		private var pClk:Clock;//  To detect time out	
		private var pProgressType:String;// PROGRESS_BY_FILE or PROGRESS_BY_BYTE
		/**
		*  progressByFile means load progress = number of file loaded and total = total files number.
		*/
		public static const PROGRESS_BY_FILE : String ="progressByFile" ;
		/**
		*  progressByByte means load progress = number of files bytes loaded and total = total bytes of all files.
		*/
		public static const PROGRESS_BY_BYTE : String ="progressByByte" ;
		/**
		* constructor
		* @param	prt Parent sprite where skin will be loaded
		* @param	base base Url of skin files
		*/
		public function SkinLoader (prt:SpriteX,base:String="./",progressType:String="progressByFile") {		
			setClassInfo("SkinLoader",this);	
			pParent=prt;
			pBase=base;
			pProgressType=progressType ;
		}
		//get/set
		/**
		* Skin description Object - AS structure of skin xml file
		* @see net.flash_line.as3.util.xml.XmlParser
		*/
		public function get skinObject () : Object {
			return pSkin;
		}		
		/**
		* load skin xml file into skinObject
		* @param	file xml filename
		*/
		public function load (file:String) {
			pFile=file;
			var xlp= new XmlParser(file);	
			xlp.addEventListener("onParseXml",fOnParseXml);
			xlp.addEventListener("onErrorXml",fOnErrorXml);
			xlp.loadXml();
		}
		/**
		* remove listeners and Clock 
		* @return null
		*/
		public function remove() {
			removeClock();
			removeListener () ;
			return null;
		}
		//
		//
		//
		private function fOnParseXml (e:EventX) {
			pSkin=e.srcVar.tree.skin ; 
			pSkin.lastLoaded=0;			
			var timeOut:Number=numVal(pSkin.timeOut,15);			
			pClk=new Clock(fOnTimeOut,timeOut);
			loadAssetOf (pSkin.root,pParent) ;			
			
		}
		private function fOnErrorXml (e:EventX) {
			
		}
		private function loadAssetOf (obj:Object,ctnr:SpriteX) {
			var i:String;
			var f:Function;
			var subSkin:Object // sub tree of pSkin
			var a:Number;//alpha
			//var r:Number;//rotation
			var vx:Number;
			var vy:Number
			var o:Object;
			for (i in obj) {				
				o=obj[i]; 
				if (!(o is String)) { 
					vx=numVal(o.x,0);
					vy=numVal(o.y,0);
					a=numVal(o.alpha,1); 
					if (isStrSet(o.file)) {
						o.file=varValueOf (o.file);						
						var src:String=pBase+pSkin.folder+o.file ;					
						o.sprite=new LoaderSprite(ctnr,src,vx,vy);	
						o.loaderCompFunc=Delegate.pass(fOnLoadAsset,o);
						o.loaderProgFunc=Delegate.pass(fOnLoadProgress,o)
						o.loaderErrFunc=Delegate.pass(fOnLoadError,o) ;
						o.sprite.addEventListener("onLoadComplete",o.loaderCompFunc);
						o.sprite.addEventListener("onLoadProgress",o.loaderProgFunc);
						o.sprite.addEventListener("onLoadError",o.loaderErrFunc);
						o.sprite.name=strVal(o.name);					
						this.stack.push(o); 
					} else if (isStrSet(o.src)) {
						o.sprite=new SpriteX (ctnr,vx,vy,a,strVal(o.name)) ;	
						subSkin=pSkin.library[o.src];
						if (isObjSet(subSkin)) {
							loadAssetOf(subSkin,o.sprite); // recursive calling
						} else {
							error("SkinLoader.loadAssetOf() : Dead relation for "+strVal(o.name,"unknow asset row in ")+pFile);
						}					
						
					} else {
						error("SkinLoader.loadAssetOf() : No file and no source for "+strVal(o.name,"unknow asset row in ")+pFile);
					}
					o.sprite.rotation=numVal(o.rotation,0);						
					if (strVal(o.visible,"true")=="false") o.sprite.visible=false;					
					o.sprite.alpha=a;
					
				}
			}
		}
		/*
		private function fOnLoadProgress (e:EventX,o:Object) {
			var loaded:Number;var total:Number;
			loaded=e.srcVar.loaded;total=e.srcVar.total;
			o.loaded=numVal(o.loaded,0) ; o.total=numVal(o.total,0) ; 
            o.loaded+=loaded ; o.total+= total;
			pSkin.loaded=numVal(pSkin.loaded,0) ; pSkin.total=numVal(pSkin.total,0) ; 
			pSkin.loaded+=loaded ; pSkin.total+=total;
			dispatchEvt(EventX.ON_LOAD_PROGRESS,{pSkin.total,loaded:pSkin.loaded});
		}
		*/
		private function fOnLoadProgress (e:EventX,o:Object) {
			var loaded:Number;var total:Number;
			loaded=e.srcVar.loaded;total=e.srcVar.total;
			loaded=numVal(loaded,0) ; total=numVal(total,0) ;
			o.loaded=numVal(o.loaded,0) ; o.total=numVal(o.total,0) ;			 
			var addLoaded=loaded-o.loaded;
			o.loaded=loaded ; 	o.total= total;
			pSkin.loaded=numVal(pSkin.loaded,0) ; pSkin.total=numVal(pSkin.total,0) ; 
			if (boolVal(o.firstProgress,true) && numVal(total,0)>0) {
				pSkin.total+=total;
				o.firstProgress=false;
			}
			pSkin.loaded+=addLoaded ;
			//
			if (pProgressType==PROGRESS_BY_BYTE) dispatchEvt(EventX.ON_LOAD_PROGRESS,{total:pSkin.total,loaded:pSkin.loaded}) ;
			else {
				loaded=0; total=this.stack.length;
				for (var i=0;i<total;i++) {
					if (boolVal(stack[i].isLoaded,false) ) loaded++;
				}
				dispatchEvt(EventX.ON_LOAD_PROGRESS,{total:total,loaded:loaded});
			}
		}
		private function fOnLoadError (e:EventX,o:Object) {
			//TO DO
		}
		private function fOnLoadAsset (e:EventX,o:Object) {
			var isLoaded:Boolean=true;			
			o.isLoaded=true ;
			for (var i:String in stack) {
				if (!boolVal(pStack[i].isLoaded) ) isLoaded=false ;
			}
			if (isLoaded == true ) {				
				removeClock();	
				removeListener();
				pClk=null;	
				new Delay2(.3).addEventListener("onDelay",fOnLoadAsset2);
				
			}			
		}
		private function fOnLoadAsset2 (e:EventX) {
			setDepthAndSize (pSkin.root,pParent) ;
			dispatchEvt("onLoadSkin");
		}
		// lev is only used to test
		private function setDepthAndSize (obj:Object,ctnr:SpriteX,lev:Number=0) {
			//alert("setDepth start lev="+lev);
			var i:String;
			var subSkin:Object // sub tree of pSkin
			var o:Object; var arr:Array = [];
			for (i in obj) {				
				o=obj[i]; 
				if (!(o is String)) { 
					if (isNumSet(o.depth)) {						
						if (o.sprite.parent == ctnr) {
							// store in tmp arr before depth setting
							arr[numVal(o.depth)] = o.sprite ;
						}						
					}
					o.sprite.height=numVal(o.height,o.sprite.height);	
					o.sprite.width=numVal(o.width,o.sprite.width);	
					if (isStrSet(o.src)) {						
						subSkin=pSkin.library[o.src];
						if (isObjSet(subSkin)) {
							setDepthAndSize(subSkin,o.sprite,lev+1); // recursive calling
						} 						
					}
					
				}
			}
			// set depths in arr order
			var len:Number = arr.length ;
			for (var j:Number = 0; j < len; j++) { 
				if (ctnr==arr[j].parent) ctnr.setChildIndex(arr[j], j);
			}
		}
		private function removeListener () {
			for (var i:String in stack) {
				stack[i].sprite.removeEventListener("onLoadComplete",stack[i].loaderCompFunc);
				stack[i].sprite.removeEventListener("onLoadProgress",stack[i].loaderProgFunc);
				stack[i].sprite.removeEventListener("onLoadError",stack[i].loaderErrFunc);
			}
		}
		private function removeClock() {
			if (pClk!=null) {
				pClk=pClk.remove();	
			}
		}
		private function fOnTimeOut (e:Event) {
			if (pSkin.loaded!=pSkin.lastLoaded) {
				pSkin.lastLoaded=pSkin.loaded ;
			} else {
				for (var i:String in stack) {			
					if (!boolVal(pStack[i].isLoaded) ) {
						error(" File not loaded="+pStack[i].file);
					}
				}
				removeClock();	
			}
			
		}		
		private function varValueOf (vn:String):String {
			if (isObjSet(pSkin.vars) ) {
				if (isObjSet(pSkin.vars[vn]) ) return strVal(pSkin.vars[vn].val) ;
			}
			return vn;
		}
		
		
	}//End of Class SkinLoader
}
