﻿/**
* @author Jean-Michel Delettre (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.io.progress {
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.display.Loader;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.ProgressEvent;
	import flash.text.TextField;
	import flash.xml.XMLParser;
	import net.flash_line.as3.util.event.EventX;
	import net.flash_line.as3.util.event.Pause;
	import net.flash_line.as3.util.global.*;
	import net.flash_line.as3.util.io.LoaderSprite;
	import net.flash_line.as3.util.motion.BasicTween;
	import net.flash_line.as3.util.xml.XmlParser;
	//
	/**
     * Dispatched when model xml file is complete loaded
     */
    [Event( name="onLoadModel", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when an error occure during model xml file loading .
     */
    [Event( name = "onErrorModel", type = "net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when the progressBar swf skin is completly loaded.
     */
    [Event( name = "onLoadSwfSkin", type = "net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when an error occure during the progressBar swf skin loading.
     */
    [Event( name = "onLoadSwfSkin", type = "net.flash_line.as3.util.event.EventX" )]
	
	public class ProgressBarMng extends EventDispatcher {
		// Properties		
		protected var _ctnr:DisplayObjectContainer;
		protected var _modelUrl:String;
		protected var _baseUrl:String;
		protected var _loadedObject:Object;
		protected var _errorModelMsg:String;
		protected var _model:Object;
		protected var _eventType:String;
		protected var _skin:DisplayObjectContainer;
		protected var _skinIsLoaded:Boolean;
		protected var _alpha:Number;
		protected var skinLS:LoaderSprite;
		protected var modelXlp:XmlParser;
		protected var bt:BasicTween;
		
		
		//
		/**
		* Define value of the eventType of this class : "progress".
		* this type mean the loadedObject dispatch ProgressEvent.PROGRESS
		*/
		public static const PROGRESS 				: String =	"progress" ;
		/**
		* Define value of the type of this class : "onLoadProgress".
		* this type mean the loadedObject dispatch EventX.ON_LOAD_PROGRESS
		*/
		public static const ON_LOAD_PROGRESS 		: String =	"onLoadProgress" ;
		/**
		 * Define value of hide/show skin tween delay
		 */
		public static const TWEEN_DELAY 		: Number =	.3 ;
		
		
		/**
		 * Constructor
		 * @param	ctnr		Will be the parent container of loaded progressBar swf
		 * @param	modelUrl	Url of Model xml file		 
		 * @param	evtTyp		progress event type. May be equal to constants : this.PROGRESS or this.ON_LOAD_PROGRESS
		 * @param	baseUrl		base url of application
		 * @param	errorModelMsg Message in case of model load error
		 */
		public function ProgressBarMng  ( ctnr:DisplayObjectContainer=null,modelUrl:String="custom/default/progressBarModel.xml",evtTyp:String="progress",baseUrl:String="",errorModelMsg:String="Progress Model can't be read !") {
			_ctnr = ctnr;
			_modelUrl = modelUrl;
			_baseUrl = baseUrl;
			_errorModelMsg = errorModelMsg;
			_eventType = evtTyp;
		}
		//gets/set
		/**
		 * Container of progressBar
		 */
		public function get ctnr () : DisplayObjectContainer {
			return _ctnr
		}
		public function set ctnr (doc: DisplayObjectContainer)  {
			_ctnr=doc
		}
		/**
		 * skin container of progressBar
		 */
		public function get skin () : DisplayObjectContainer {
			return _skin ;
		}
		public function set skin (doc: DisplayObjectContainer)  {
			_skin=doc
		}
		/**
		 * Object (swf,stream,BatchLoader,etc) which dispatch ProgressEvent.PROGRESS event .
		 */
		
		public function get loadedObject () : Object {
			return _loadedObject ;
		}
		public function set loadedObject (o: Object)  {
			_loadedObject = o;
		}
		/**
		* progress event type. May be equal to constants : this.PROGRESS or this.ON_LOAD_PROGRESS
		* @see ProgressBarMng.PROGRESS
		* @see ProgressBarMng.ON_LOAD_PROGRESS
		*/
		public function get eventType () : String {
			return _eventType ;
		}
		public function set eventType (s:String)  {
			_eventType = s;
		}
		/**
		 * Base url of application
		 */
		public function get baseUrl () : String {
			if (_baseUrl == null) _baseUrl = "";
			return _baseUrl
		}
		public function set baseUrl (s: String)  {
			_baseUrl = s;
		}
		/**
		 * Url of this class's model xml file
		 */
		public function get modelUrl () : String {
			return _modelUrl
		}
		public function set modelUrl (s: String)  {
			_modelUrl = s;
		}
		/**
		 * error msg if modelUrl file doesn't exist or invalid
		 */
		public function get errorModelMsg () : String {
			return _errorModelMsg
		}
		public function set errorModelMsg (s: String)  {			
			_errorModelMsg = s;
		}
		/**
		* image Object of the model xml file :
		* @see ./progress/sample/progressBarModel.xml 
		*/
		public function get model () : Object {
			return _model ;
		}
		/**
		 * visible progress bar. Its width is relative from loaded bytes %
		 */
		public function get progressBar () : DisplayObject {
			return skin.getChildByName(model.bar.name) ;
		}
		/**
		 * textField where loaded % is displayed (facultaive)
		 */
		public function get percentTf () :TextField {
			return TextField(skin.getChildByName(model.percentTf.name) );
		}
		/**
		 * textField where a title is displayed (facultaive)
		 */
		public function get labelTf () :TextField {
			return TextField(skin.getChildByName(model.labelTf.name) );
		}
		/**
		 * true if skin is not visible.
		 */
		public function get skinIsHidden () :Boolean {
			if (skin != null) return (!skin.visible ) ;
			else return true ;
			
		}
		/**
		 * true if model and skin loading is finished correctly
		 */
		public function get skinIsLoaded () : Boolean {
			_skinIsLoaded = boolVal(_skinIsLoaded, false);
			return (_skinIsLoaded && skin != null) ;
		}
		public function set skinIsLoaded (b: Boolean)  {
			_skinIsLoaded = b;
		}
		/**
		 * true if model loading is finished correctly
		 */
		public function get modelIsLoaded () : Boolean {			
			return (model!=null) ;
		}
		public function get alpha () : Number {
			if (numVal(_alpha,Number.NEGATIVE_INFINITY) == Number.NEGATIVE_INFINITY ) _alpha = 1;
			return _alpha ;
		}
		public function set alpha (n:Number)  {
			_alpha = n;
		}
		public function set title (s:String)  {
			if (title != null);
			model.lang.downloadTitle.label = s;
		}
		public function get title () : String {
			if (model.lang.downloadTitle!=null) return model.lang.downloadTitle.label ;
			return null;
		}
		
		
		
		//------Public method ------ 
		/**
		 * Basic process entry of this class.
		 * Call this method :
		 * 1_ after assignation of all needed parameters or using the Model xml file which contains the parameters.
		 * 2_ after to have launched the loadedObject loading.
		 * If startProgress was already called one, the xml model -end skin- will not be loaded again ;
		 * but if the parameter modUrl is not null the xml model -and skin- will be re-loaded.
		 * @param	lo The loaded Object which dispatch a PROGRESS  event type
		 * @param	modUrl xml model url
		 * @param	evtTyp progress event type. May be equal to constants : this.PROGRESS or this.ON_LOAD_PROGRESS
		 * @see ctnr() 
		 * @see loadedObject()
		 * @see eventType()
		 * @see modelUrl()
		 * @see baseUrl()
		 * @see ./progress/sample/progressBarModel.xml 
		 */
		public function startProgress (lo: Object = null,evtTyp:String=null, modUrl:String = null) {
			if (strVal(evtTyp, "") != "") eventType = evtTyp;
			if (strVal(modUrl, "") != "") modelUrl = modUrl;
			if (lo!=null) loadedObject = lo;
			//
			if (strVal(modUrl, "") != "" || !modelIsLoaded) {				
				addEventListener("onLoadSwfSkin", startProgress2);
				loadModel();
			} else {
				createProgressListener();
			}
		}
		public function startProgress2 (e:Event) {
			removeEventListener("onLoadSwfSkin", startProgress2);
			createProgressListener();
		}
			
		/**
		 * Hide the visible progress bar and stop progression listening
		 * but keep ready instance of this class to be used with an other future loadedObject 
		 */
		public function stopProgress() {	
			hide();
			removeProgressListener();
		}
		/**
		 * remove and free memory instance of this class.
		 * @return Null
		 */
		public function remove() : Object {	
			stopProgress();
			//removeLoadSwfSkinListener ();
			//removeLoadModelListener ();			
			if (skinIsLoaded) {				
				LoaderSprite(skin).remove();
				_skin = null;	
			}			
			_model = null;
			_loadedObject = null;	
			return null;
			
		}
		/**
		 * Hide progressBar's skin 
		 */
		public function hide() {
			var d:Number=ProgressBarMng.TWEEN_DELAY;
			if (skin != null && !skinIsHidden) {
				if (bt!=null) bt.remove();
				bt = new BasicTween(skin, "alpha" , 0, d);	
				new Pause(hide2, d);
			}
		}
		private function hide2(e:Event) {
			if (skin != null ) skin.visible = false;
		}
		/**
		 * show progressBar's skin 
		 */
		public function show () {	
			var d:Number=ProgressBarMng.TWEEN_DELAY;
			if (skin != null && skinIsHidden) {
				skin.visible = true; skin.alpha = 0;
				if (bt!=null) bt.remove();
				bt = new BasicTween(skin,"alpha" ,alpha, d);				
			}
		}
		/**
		 * load xml Model file in this.model Object
		 * @param	str The url of xml file
		 */
		public function loadModel(str:String=null)  {
			if (strVal(str, "") != "") modelUrl = str;
			var src:String = getFullPathFrom(modelUrl, baseUrl);
			modelXlp= new XmlParser(src);	
			modelXlp.addEventListener(EventX.ON_PARSE_XML,onLoadModel);
			modelXlp.addEventListener(EventX.ON_ERROR_LOAD_XML,onErrorModel);
			modelXlp.loadXml();
		}
		/**
		 * remove model xml file's XmlParser listeners.
		 * This method is automaticly called when model xml file is loaded.
		 */
		public function removeLoadModelListener () {
			if (modelXlp!=null) {
				modelXlp.removeEventListener(EventX.ON_PARSE_XML,onLoadModel);
				modelXlp.removeEventListener(EventX.ON_ERROR_LOAD_XML, onErrorModel);
			}
		}
		/**
		 * Load skin as an external swf
		 * @param	str swf Url
		 * @param	vx y location
		 * @param	vy y location
		 */
		public function loadSwfSkin(str:String=null,vx:*=null,vy:*=null)  {
			if (strVal(str, "") != "") model.skin.src = str;
			var src:String = getFullPathFrom(model.skin.swf, baseUrl);
			//
			if (vx != null) model.skin.x = numVal(vx, 0).toString();
			if (vy != null) model.skin.y = numVal(vy, 0).toString();
			if (boolVal(model.skin.centered,true)) {
				vx = Number(ctnr.stage.stageWidth / 2) ; 
				vy = Number(ctnr.stage.stageHeight / 2) ;
			} else {
				vx = numVal(model.skin.x, 0);
				vy = numVal(model.skin.y, 0);
			}
			//
			var c:DisplayObjectContainer;
			if (boolVal(model.skin.childOfStage, true)) c = ctnr.stage ;
			else c = ctnr;
			//
			skinLS=new LoaderSprite(c,src,vx,vy);
			skinLS.addEventListener(EventX.ON_LOAD_COMPLETE, onLoadSwfSkin);
			skinLS.addEventListener(EventX.ON_LOAD_ERROR, onErrorSwfSkin);
		}
		/**
		 * Remove skin swf's XmlParser listeners.
		 * This method is automaticly called when skin swf file is loaded.
		 * @param	xlp
		 */
		public function removeLoadSwfSkinListener() {
			if (skinLS != null) {
				skinLS.removeEventListener(EventX.ON_LOAD_COMPLETE, onLoadSwfSkin);
				skinLS.removeEventListener(EventX.ON_LOAD_ERROR, onErrorSwfSkin);
			}
		}
		/**
		 * Add progress listener on loadedObject
		 * @param	lo loaded Object
		 * @param	eventType ProgressBarMng.PROGRESS or ProgressBarMng.ON_LOAD_PROGRESS or ...
		 */
		public function createProgressListener(lo:Object=null,evtTyp:String=null) {
			if (lo != null) loadedObject = lo;
			if (evtTyp != null) eventType = evtTyp;
			if (loadedObject!=null) {
				if (eventType==ProgressBarMng.PROGRESS) {
					loadedObject.addEventListener(ProgressEvent.PROGRESS, onProgress);
				} else if (eventType == ProgressBarMng.ON_LOAD_PROGRESS) {
					loadedObject.addEventListener(EventX.ON_LOAD_PROGRESS, onLoadProgress);
				} else {
					error(model.error.eventTypeError.label);
				}
			} else {
				error(model.error.loadedObjectError.label);
			}
		}
		/**
		 * remove progress event listeners
		 */
		public function removeProgressListener() {
			if (eventType==ProgressBarMng.PROGRESS) {
				if (loadedObject!=null) loadedObject.removeEventListener(ProgressEvent.PROGRESS, onProgress);
			} else if (eventType==ProgressBarMng.ON_LOAD_PROGRESS) {
				if (loadedObject!=null) loadedObject.removeEventListener(EventX.ON_LOAD_PROGRESS, onLoadProgress);
			} else {
				if (model!=null) error(model.error.eventTypeError.label);
			}
		}
		/**
		 * Display progression of the bar
		 * @param	loaded Current loaded bytes number of loadedObject.
		 * @param	total Total bytes number of loadedObject.
		 */
		public function display(loaded:Number,total:Number) {			
			if ( loaded>0) {			
				if (skinIsLoaded && skinIsHidden) show();					
				if (total>0) {	
					// bar
					progressBar.scaleX = loaded / total ;
					// percent
					if (percentTf != null) {
						var percentValue:Number = Math.round( loaded / total*100) ;
						if (strVal(model.percentTf.size,"short")=="short") {
							percentTf.text = percentValue + " %" ;
						} else if (strVal(model.percentTf.size, "short") == "long") {							
							var unit:Number	= 1000 ;
							var unitTxt:String 	= " kb";		
							if (total < 1000) {
								unit 		= 1 ;
								unitTxt 	= " bytes";
							} 
							percentTf.text = model.lang.total.label+Math.floor(total/unit)+unitTxt+model.lang.total.label+Math.floor(loaded/unit)+unitTxt
						} 
					}
					// end progress test
					if (loaded == total) stopProgress();
				}
				// title
				if (strVal(model.lang.downloadTitle.label,"")!="") {
					if (labelTf!=null) {
						labelTf.text = model.lang.downloadTitle.label ;
					}
				}  
			}
		}		
		/**
		* @private methods
		*/		
		private function onErrorModel(e:Event) {
			removeLoadModelListener ();
			error(errorModelMsg);	
			dispatchEvent(new Event("onErrorModel"));
		}
		private function onLoadModel (e:Event):void {	
			removeLoadModelListener ();
			_model = e.target.tree;
			_alpha = numVal(model.skin.alpha, 1) ;
			loadSwfSkin();
			skin = skinLS;
			skin.visible=false;
			dispatchEvent(new Event("onLoadModel"));
		}
	    private function onLoadSwfSkin (e:Event):void {	
			skinIsLoaded = true;
			removeLoadSwfSkinListener();
			dispatchEvent(new Event("onLoadSwfSkin"));
		}
	    private function onErrorSwfSkin(e:Event) {
			removeLoadSwfSkinListener();
			error(model.error.skinLoadError.label);	
			dispatchEvent(new Event("onErrorSwfSkin"));
		}
		private function onProgress(e:ProgressEvent) {	
			display(e.bytesLoaded, e.bytesTotal);
			dispatchEvent(new Event("onProgress"));
		}
		private function onLoadProgress(e:EventX) {	
			display(e.srcVar.loaded, e.srcVar.total);
			dispatchEvent(new Event("onLoadProgress"));
		}
		
		
	}
}
		
		