﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.io {
	import net.flash_line.as3.util.common.* ;
	import net.flash_line.as3.util.global.* ;
	import flash.events.* ;
	import flash.net.* ;
	//
	/**
     * Dispatched when data are correctly sent and loaded
	 * @eventType net.flash_line.as3.util.event.EventX.ON_SEND_AND_LOAD
	 * @see net.flash_line.as3.util.common.Super#dispatchEvt()
     */
    [Event( name="onSendAndLoad", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when when data are correctly loaded
	 * @eventType net.flash_line.as3.util.event.EventX.ON_LOAD
	 * @see net.flash_line.as3.util.common.Super#dispatchEvt()
     */
    [Event( name="onLoad", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when when xml are correctly loaded
	 * @eventType net.flash_line.as3.util.event.EventX.ON_LOAD_XML
	 * @see net.flash_line.as3.util.common.Super#dispatchEvt()
     */
    [Event( name="onLoadXml", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when an error occure before during data loading
	 * @eventType net.flash_line.as3.util.event.EventX.ON_ERROR_LOAD_DATA
	 * @see net.flash_line.as3.util.common.Super#dispatchEvt()
     */
    [Event( name="onErrorLoadData", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when an error occure before during xml loading
	 * @eventType net.flash_line.as3.util.event.EventX.ON_ERROR_LOAD_XML
	 * @see net.flash_line.as3.util.common.Super#dispatchEvt()
     */
    [Event( name="onErrorLoadXml", type="net.flash_line.as3.util.event.EventX" )]
	/**
	* A set of I/O methods - server exchanges
	*/ 
	public class Xdata extends Super  { 		
		/**
		* @private
		* externe file/program to xchange with
		*/
		protected var pSource:String			;	
		/**
		* @private
		* basic error message during test
		*/
		protected var pErrMsg:String=null			;	
		/**
		* @private
		* UrlLoader
		*/
		protected var pUld:URLLoader
		/**
		* constructor
		*/
		public function Xdata() {  
			setClassInfo("Xdata",this);
	   } 	  
		/**
		* simple xml load without data sent
		* @param	src Xml Url
		* @param	errMsg String != null if an error message must be displayed by Xdata itself 
		*/	  
		public function loadXml(src:String,errMsg:String=null) {		
			pSource = src ;	
			if (errMsg!=null) pErrMsg =errMsg ;
			var req:URLRequest = new URLRequest(src);
           if (pUld==null) {
				pUld = new URLLoader();
			} else {
				remove ();
			}			
            pUld.addEventListener(IOErrorEvent.IO_ERROR, fOnError);
			pUld.addEventListener(Event.COMPLETE, fOnLoadXml);			
			pUld.load(req); 
			          
		}
		/**
		* Send vars to Url and receive data back
		* @param	src Url
		* @param 	vars Object with vars to send
		* @param	errMsg String != null if an error message must be displayed by Xdata itself 
		*/	  
		public function sendAndLoadVars(src:String,vars:Object=null,errMsg:String=null,method:String="POST") {		
			var loadOnly:Boolean=(vars==null);
			pSource = src ;	
			pErrMsg =errMsg ;
			//
			var str:String="";
			if (!loadOnly) {
				var uv:URLVariables=new URLVariables();
				for (var i:String in vars) {					
					uv[i]=vars[i];
				}
			}
			//
			var req:URLRequest = new URLRequest(src);
			req.method = method;
			if (!loadOnly) req.data = uv ;
            if (pUld==null) {
				pUld = new URLLoader();
			} else {
				remove ();
			}
			pUld.dataFormat = URLLoaderDataFormat.VARIABLES;
            pUld.addEventListener(IOErrorEvent.IO_ERROR, fOnError);
			pUld.addEventListener(Event.COMPLETE,fOnSendAndLoad);  
			pUld.load(req);            
		}
		/**
		* Receive data only from an Url
		* @param	src Url
		* @param	errMsg String != null if an error message must be displayed by Xdata itself 
		*/	  
		public function loadVars(src:String,errMsg:String=null) {		
			 sendAndLoadVars(src,null,errMsg)   ;   
		}
		
		/**
		* send vars to Url and receive Xml back
		* @param	src Url
		* @param 	vars Object with vars to send
		* @param	errMsg String != null if an error message must be displayed by Xdata itself
		*/	  
		public function sendVarAndLoadXml(src:String,vars:Object,errMsg:String=null) {		
			pSource = src ;	
			pErrMsg =errMsg ;
			//
			var str:String="";
			var uv:URLVariables=new URLVariables();
			for (var i:String in vars) {
				uv[i]=vars[i];
			}
			//
			var req:URLRequest = new URLRequest(src);
			req.method = URLRequestMethod.POST;
			req.data = uv ;
           if (pUld==null) {
				pUld = new URLLoader();
			} else {
				remove ();
			}
			//pUld.dataFormat = URLLoaderDataFormat.VARIABLES;
            pUld.addEventListener(IOErrorEvent.IO_ERROR, fOnError);
			pUld.addEventListener(Event.COMPLETE,fOnLoadXml);  
			pUld.load(req);            
		}
		
		public function callUrl (url:String,winType:String="_blank"):void {
			var req:URLRequest = new URLRequest(url);
			navigateToURL(req,winType);
        }

		/**
		* remove URLLoader's listeners
		*/
		public function remove ():void {
			if (pUld.hasEventListener(IOErrorEvent.IO_ERROR)) pUld.removeEventListener(IOErrorEvent.IO_ERROR, fOnError);
			if (pUld.hasEventListener(Event.COMPLETE)) {
				pUld.removeEventListener(Event.COMPLETE,fOnLoadXml);
				pUld.removeEventListener(Event.COMPLETE, fOnSendAndLoad);
			}
        }
		//
		//
		// private methods
		//
		//
		/**
		* listen URLLoader.load() . Okay for many methods except loadXml()
		* @param	event Event instance
		* @event 	"onLoadData"
		* @see fOnLoadXml() 
		*/
		private function fOnSendAndLoad(event:Event):void {						
				var ul:URLLoader = URLLoader(event.target);
				var uv:URLVariables = new URLVariables(ul.data);
				dispatchEvt("onSendAndLoad",{data:uv,instance:this} );
				dispatchEvt("onLoad",{data:uv,instance:this} );
				remove();

			
        }

		/**
		* listen URLLoader.load()
		* @param	event Event instance
		* @event 	"onLoadXml"
		*/
		private function fOnLoadXml(event:Event):void {				
			var x:XML = new XML(event.target.data);
			dispatchEvt("onLoadXml",{xml:x,instance:this} ); 
			remove();
        }
		/**
		* listen URLLoader.load() if error
		* @param	event Event instance
		* @event 	"onErrorLoadXml"
		* @event 	"onErrorLoadData"
		*/
		private function fOnError(event:Event):void {
			dispatchEvt("onErrorLoadData",{error:"loadError" } );
			dispatchEvt("onErrorLoadXml",{error:"loadError" } );
			if (pErrMsg!=null) error(pErrMsg) ;
			remove();
        }
		
		
		
	}
}
