﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.motion {
	
	import flash.events.* ;
	import flash.display.* ;
	import net.flash_line.as3.util.common.Super ;
	import net.flash_line.as3.util.event.* ;
	import fl.transitions.Tween;
	import fl.transitions.easing.*;
	import fl.transitions.TweenEvent ;
	/**
     * Dispatched when the tween is finished
	 * @eventType net.flash_line.as3.util.event.EventX.ON_END_TWEEN
	 * @see net.flash_line.as3.util.common.SuperX#dispatchEvt()
     */
    [Event( name="onEndTween", type="net.flash_line.as3.util.event.EventX" )]
	/**
	* Most frequently used tween type.
	* <p>We pass only end tween value; not start value. <br/>
	* The Function is by default Regular.easeInOut<br/>
	* and duration 0.5 second.</br>
	* So we just instance as this : new BasicTween(mySprite,"property",endValue);<br/>
	* BasicTween doesn't extend Tween but instance one </p>
	*/
	public class BasicTween extends Super {	
		protected var pTween:Tween;
		protected var pMc:DisplayObject;
		/**
		* constructor - launch Tween
		* @param	mc		tweened clip
		* @param	prop	property
		* @param	e		end value
		* @param	d		duration
		* @param	f		easing function
		*/
		public function BasicTween(mc:DisplayObject,prop:String,e:Number,d:Number=.5,f:Function=null,sec:Boolean=true) {
			pMc=mc;
			if (isNaN(e) || mc==null){
				error("Error in BasicTween : \ne="+e+"\nmc="+mc+"\nprop="+prop);
			}
			var s=mc[prop] ;
			if (f==null) f=Regular.easeInOut;
			pTween = new Tween(mc,prop,f,s,e,d,sec);
			pTween.addEventListener(TweenEvent.MOTION_FINISH, onEnd);	
			pTween.addEventListener(TweenEvent.MOTION_CHANGE , onChange);	
			
			setClassInfo("BasicTween",this);
		}
		/**
		* the Tween instance
		*/
		public function get tween () : Tween {
			return pTween ;
		}
		/**
		* the target 
		*/
		public function get mc () : DisplayObject {
			return pMc ;
		}
		/**
		* remove Tween listener
		*/
		public function remove() {
			if (pTween!=null) {
				pTween.removeEventListener(TweenEvent.MOTION_FINISH,onEnd);	
				pTween.stop();
			}			
			pTween=null;
			return null;
		}
		/**
		* alias of remove() to test
		*/
		public function clear() {
			//alert("clear in BT");
			return remove();				
		}
		//
		// private
		//
		/**
		* listen TweenEvent.MOTION_FINISH
		* @param	e event object
		* @dispatch "onEndTween" 
		* @see net.flash_line.as3.util.event.EventX
		*/
		private function onEnd(e:Event) {
			
			dispatchEvt("onEndTween",{tween:this});
			remove() ;
		}
		private function onChange(e:Event) {			
			dispatchEvt(EventX.ON_CHANGE_TWEEN,{tween:this});
		}



	}
}

