﻿/**
* Abstract MVC Model class
* @author Jean-Michel Delettre (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.mvc {	
	
	import net.flash_line.as3.util.xml.XmlParser ;
	import net.flash_line.as3.util.common.Super ;
	import net.flash_line.as3.util.event.* ;
	//
	public class AbstractModel extends Super { 
		private var pTree:Object ; // model tree
		private var pMain:*;
				
		/**
		* constructor
		* @param	mn Main instance - caller
		*/
		public function AbstractModel (mn:*) {		
			pMain=mn;
			setClassInfo("AbstractModel",this);	
		}
		/*** get/set ****/
		/**
		* @return Main instance
		*/
		public function get _main ():* {
			return pMain;
		}
		/**
		* Receveid flashVars Object 
		*/
		public function get receivedVar (): Object {
			return pMain.receivedVar;
		}				
		/**
		* 
		* @return standard tween duration
		*/
		public function get tweenDur ():Number {
			return Number(tree.application.tweenDuration);
		}
		/**
		* Software BASE url.
		* come ever from html received var or from model xml file.
		*/
		public function get baseUrl ():String {
			if (pMain.baseUrl!=null) return pMain.baseUrl ; 
			else return tree.application.baseUrl;
		}
		/**
		* 
		* @return xml folder
		*/
		public function get xmlFolder ():String {
			if (tree.folder!=null) {
				if (tree.folder.xml!=null) return baseUrl+tree.folder.xml;					
			}
			return baseUrl;
		}
		/**
		* 
		* @return skin xml filename
		*/
		public function get xmlSkin ():String {
			return xmlFolder+tree.skin.src;
		}
		/**
		* 
		* @return true if appli is with xml defined skin
		*/
		public function get isWithSkin ():Boolean {
			return (tree.skin!=null) ;
		}		
		/**
		* 
		* @return basic model tree
		*/
		public function get tree ():Object {
			return pTree;
		}
		
		/**
		* Alias of tree
		*/
		public function get param ():Object {
			return tree;
		}
		/**
		* 
		* @return application's framerate
		*/
		public function get frameRate ():Number {
			if (isNaN(pTree.application.frameRate)) return 60 ;
			else return Number(pTree.application.frameRate);
		}
		/**
		* load Model from xml file
		* @param	initUrl xml model filename
		*/
		public function loadModel (initUrl:String):void {
			var xlp= new XmlParser(initUrl);	
			xlp.addEventListener("onParseXml",fOnLoadModel);
			xlp.addEventListener("onErrorXml",Delegate.pass(fOnErrorModel,initUrl));
			xlp.loadXml();
		}
		public function getFullPath (src:String,base:String=null):String {			
			if (src.substring(0,4)!="http") {
				if (base!=null) return base+src;
				if (baseUrl!=null) return baseUrl+src;
				else return src;
			} else return src
		}
		
		/**
		* Listen LoadModel if load error
		* @param	e custom Event
		* @param	initUrl Filename
		*/	
		private function fOnErrorModel (e:EventX,initUrl:String) {
			error("Can't read file : "+initUrl);		
		}
		/**
		* Listen LoadModel if Okay
		* @param	tree Structured AS tree
		* @param	xlp 
		*/		
		private function fOnLoadModel (e:EventX):void {		
			//alert("e.srcVar.tree="+e.srcVar.tree);
			pTree=e.srcVar.tree;
			//
			dispatchEvt("onLoadModel");			
		}
		
	}//End of Class AbstractModel
}//end of package
