﻿/**
* net.flash_line.as3.util.mvc contains abstract MVC class
* @author Jean-Michel Delettre (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/
package net.flash_line.as3.util.mvc {	

	import net.flash_line.as3.util.contextHelp.custom.WhoRightClic ;
	import net.flash_line.as3.util.common.SpriteX;
	import net.flash_line.as3.util.global.*;
	/**
	* Abstract initial class , instanciate MVC classes 
	* @see AbstractModel
	* @see AbstractView
	* @see AbstractControler
	*/ 
	public class   AbstractMain extends SpriteX { 
		var pModelClass:Class;
		var pViewClass:Class;
		var pControlerClass:Class;
		//
		protected var pModelSrc:String;
		var pBaseUrl:String;
		protected var pProgressSrc:String;
		var pModel:*;
		var pView:*;
		var pControl:*;
		/**
		* 
		* @param	prt		The root sprite of application  - generaly the root caller itself
		* @param	mc		No abstract Model class
		* @param	vc		No abstract View class
		* @param	cc		No abstract Controler class
		*/
		public function AbstractMain (prt:SpriteX,mc:*,vc:*,cc:*) {				
			super(prt,0,0,1,"mainSprite");			
			pParent=this;				
			pParent.stage.scaleMode = "noScale";
			pParent.stage.align = "TL";				
			// records instance 
			setClassInfo("AbstractMain",this);
			pModelClass=mc;
			pViewClass=vc
			pControlerClass=cc;
		}
		
		/**
		* Model subclass of AbstractModel
		*/
		public function get model () :* {
			return pModel;
		}
		/**
		* Controler subclass of AbstractControler
		*/
		public function get control () :* {
			return pControl;
		}
		/**
		* View subclass of AbstractView
		*/
		public function get view () :* {
			return pView;
		}
		/**
		* Library class to access to some embeded assets
		*/
		public function get embedLib ():* {
			return pCtnr;
		}
		/**
		* Software BASE url
		*/
		public function get baseUrl ():String {
			return pBaseUrl;
		}
		/**
		* create MVC instances and launch control.begin()
		* @param begin If true Control.begin() is launched automaticly
		* @see #control
		* @see AbstractControler#begin()
		*/
		public function create (begin:Boolean = true):void {
			pModel=new pModelClass(this);			
			pView=new pViewClass(this,pParent);			
			pControl = new pControlerClass(this, pModel, pView);
			if (begin) pControl.begin(pModelSrc,pProgressSrc);			
		}
		/**
		* May be overrided by sub classes
		* @param	version 
		*/
		public function init (version:String,vName:String,font:String="Verdana",size:Number=12):void {
			// use only in subclass : if component(s) exist : setGlobalStyle(font,size);
			new WhoRightClic(this,vName+" "+version);	
			appInit (pParent,font,size);
		}
		/**
		* 
		* @param	vmf name of received variable which contains model filename
		* @param	def Model filename default
		*/
		public function setModelFilename (vmf:String="SOURCE",def:String="xml/model.xml"):void {
			pModelSrc = pParent.receivedVar[vmf]; 
			if (pModelSrc==null) pModelSrc=def;
			if (pBaseUrl!=null) pModelSrc=pBaseUrl+pModelSrc;
		}
		/**
		* 
		* @param	vmf name of received variable which contains the software base url
		* @param	def soft base url by default
		*/
		public function setBaseUrl (vmf:String="baseUrl",def:String=""):void {
			pBaseUrl = pParent.receivedVar[vmf]; 
			if (pBaseUrl==null) pBaseUrl=def;
		}
		public function setProgressFilename (pf:String="asset/progress.swf"):void {
			pProgressSrc = pf; 
		}
		
		
	} //End of Class AbstractMain
}