﻿/**
* Manage a node -external graphic display objects- which have one NodeGroupMng child OR end elements children  -
* this NodeMng is child of a NodeGroupMng parent .
* @see net.flash_line.as3.util.graph.NodeGraph (facultative)
* @see net.flash_line.as3.util.ui.NodeGroupMng
* @author Jean-Michel Delettre (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.ui {	
	
	import flash.display.* ;
	import flash.events.Event ;
	import net.flash_line.as3.util.common.*;
	import net.flash_line.as3.util.event.*;
	//	
	public class  NodeMng  extends Super {
			var pOpen:Boolean;
			var pOpener:DisplayObjectContainer ;
			var pHead:DisplayObjectContainer	;	
			var pMsk:DisplayObjectContainer		;
			var pContainer:DisplayObjectContainer;
			var pNodeGroupMng:NodeGroupMng;
			var pArrElem:ArrayX;
			var pSp:DisplayObjectContainer;
		/**
		* Constructor
		* @param	opener		+/- open/close button
		* @param	head		header - node's always visible part
		* @param	msk			mask
		* @param	container 	sprite which is masked 
		*/
		public function NodeMng (opener:DisplayObjectContainer,head:DisplayObjectContainer,msk:DisplayObjectContainer,container:DisplayObjectContainer) { 
			pArrElem=new ArrayX();
			pOpen=false;
			pSp=head.parent;
			pOpener=opener;		
			pHead=head;		
			pMsk=msk;		
			pContainer=container;	
			setClassInfo("NodeMng",this);
			createEvent();
			
		}
		// get/set
		/**
		* 
		* @return parent sprite 
		*/
		public function get sp ():DisplayObjectContainer {
			return pSp;
		}
		public function get container ():DisplayObjectContainer {
			return pContainer;
		}
		/**
		* 
		* @return header sprite - node's always visible part
		*/
		public function get head ():DisplayObjectContainer {
			return pHead;
		}
		/**
		* 
		* @return mask sprite
		*/
		public function get msk ():DisplayObjectContainer {
			return pMsk;
		}
		public function get nodeGroupMng  ():NodeGroupMng {
			return pNodeGroupMng;
		}
		/**
		* 
		* @return visible height of this node
		*/
		public function get height ():Number {
			return head.height+msk.height;
		}
		/**
		* add listener to open/close button
		*/
		public function createEvent() {			
			pOpener.addEventListener("click",fOpenClose);
			pHead.addEventListener("click",fHeadClick);
			closeNode();
		}
		/**
		* External public open/close button - PRIVATE USE ONLY for NOW
		*/
		private function openOrClose() {	
			if (pOpen==false) openNode();			
			else closeNode();
		}	
		/**
		* open this node 
		*/
		public function openNode() {			
			pOpen=true;
			if (pNodeGroupMng!=null) {
				pMsk.height=pNodeGroupMng.height;
			} else {
				pMsk.height=pContainer.height;	
			}
			dispatchEvt("onNodeResize",{nodeMng:this,state:pOpen});
		}
		/**
		* close this node
		*/
		public function closeNode() {	
			pOpen=false;
			pMsk.height=0;			
			dispatchEvt("onNodeResize",{nodeMng:this,state:pOpen});
		}		
		/**
		* add a final element in masked container
		* @param	sp 	Sprite to add
		* @param	n		name
		*/		
		public function addElem(sp:DisplayObjectContainer) {
			var vy:Number=pContainer.height;
			pContainer.addChild(sp); 	
			sp.x=10; sp.y=vy;
			pArrElem.push(sp);
		}
		/**
		* Create  NodeGroupMng child - one only
		*/
		public function createNodeGroup() {
			removeNodeGroupMng();
			pNodeGroupMng= new NodeGroupMng (pContainer);
			pNodeGroupMng.addEventListener("onNodeGroupResize",fOnNodeGroupResize);
			return pNodeGroupMng;
		}
		/**
		* remove this and child of
		*/
		public function remove() {
			pOpener.removeEventListener("click",fOpenClose);	
			if (pNodeGroupMng!=null) {
				pNodeGroupMng.removeEventListener("onNodeGroupResize",fOnNodeGroupResize);
				pNodeGroupMng.remove();
			}
			pSp.parent.removeChild(pSp);
		}
		// Private classes
		private function removeNodeGroupMng() {				
			if (pNodeGroupMng!=null) {
				pNodeGroupMng.removeEventListener("onNodeGroupResize",fOnNodeGroupResize);
				pNodeGroupMng.remove();
			}
		}
		private function fOpenClose(e:Event) {	
			openOrClose();
		}
		private function fHeadClick(e:Event) {	
			dispatchEvt("onNodeClick",{nodeMng:this});
		}
			
		private function fOnNodeGroupResize(e:EventX) {
			if (pOpen) {
				pMsk.height=pNodeGroupMng.height;
			}
			dispatchEvt("onNodeResize",{nodeMng:this,state:pOpen});
		}
	}
}

