﻿/**
* @author Jean-Michel Delettre © (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.ui.bubble {
	import flash.display.* ;
	import flash.events.* ;
	import flash.text.*;
	import net.flash_line.as3.util.common.*;
	import net.flash_line.as3.util.global.*;
	import net.flash_line.as3.util.event.*;
	import net.flash_line.as3.util.text.TextFieldX;
	import net.flash_line.as3.util.graph.*;
	import net.flash_line.as3.util.motion.BasicTween ;
	/**
     * Dispatched when info-bubble appear
	 * @eventType net.flash_line.as3.util.event.EventX.ON_DISPLAY_BUBBLE
	 * @see net.flash_line.as3.util.common.Super#dispatchEvt()
     */
    [Event( name="onDisplayBubble", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when info-bubble disapear.
	 * @eventType net.flash_line.as3.util.event.EventX.ON_REMOVE
	 * @see net.flash_line.as3.util.common.Super#dispatchEvt()
     */
    [Event( name="onRemove", type="net.flash_line.as3.util.event.EventX" )]
	/**
	* Manage info bubble .
	* <p>It's not necessary to use public methods and properties except constructor<br/>
	* but subclasses may extend this class.</p>
	*/ 	
    public class  BubbleInfo  extends Super {
		/**
		* @private
		*/
		protected 	var pCtnr		: DisplayObjectContainer	; 	// the bubbles container area
		/**
		* @private
		*/
		protected 	var pMc			: DisplayObject		; 	// source of event
		/**
		* @private
		*/
		protected 	var pTxt		: String		;	// associate text
		/**
		* @private
		*/
		protected 	var pX			: Number		;	// pos _x 
		/**
		* @private
		*/
		protected 	var pY			: Number		;	// pos _y	
		/**
		* @private
		*/
		protected	var pW			: Number		;	// width		
		/**
		* @private
		*/
		protected	var pH			: Number		; 	// Height
		/**
		* @private
		*/
		protected 	var pDelay		:Number			;	// RollOver delay in second 
		/**
		* @private
		*/
		protected 	var pDelayTween	:Number			;	// Tween delay in second
		/**
		* @private
		*/
		protected 	var pAlpha		:Number			;	// back alpha
		/**
		* @private
		*/
		protected	var pTarget			: SpriteX			;	// info buble MC -created by this instance-
		/**
		* @private
		*/
		protected	var pTxtFld			: TextFieldX		;	// text field class
		/**
		* @private
		*/
		protected	var pDel			: Delay2			;	
		/**
		* @private
		*/
		protected	var pNoSwap			: Boolean			;	
		/**
		* Constructor
		* @param	vMc				// source of event 
		* @param	vTxt			// associate text
		* @param	vW				// width
		* @param	vH				// height
		* @param	vX				// pos x 
		* @param	vY				// pos y
		* @param	vDelay			// RollOver delay in second 
		* @param	vDelayTween		// Tween delay in second
		* @param	vAlpha			// bg alpha
		* @param	vNoSwap			// if true the bubble depth is not swaped with the source of event
		*/
		public function BubbleInfo		  ( ctnr		:DisplayObjectContainer ,
											vMc			:DisplayObject	,
											vTxt		:String		,	
											vX			:Number			=NaN,
											vY			:Number 		=NaN,
											vW			:Number			=150,
											vH			:Number			=20,
											vDelay		:Number			=1.2,
											vDelayTween :Number			=.7,
											vAlpha		:Number			= .65 ,
											vNoSwap		:Boolean		= false 
										) {		
			pCtnr		= ctnr		;
			pMc			= vMc		;
			pTxt		= vTxt		;
			pX			= vX		;
			pY			= vY		;
			pW			= vW		;
			pH			= vH		;
			pDelay		= vDelay	;	
			pDelayTween		= vDelayTween	;	
			pAlpha = vAlpha;
			pNoSwap = vNoSwap ;
			setClassInfo("BubbleInfo",this);		
			createEvent() ;
		}
		// set & get
		/**
		* The container of info-bubble
		*/
		public function get ctnr () : DisplayObjectContainer {
			return pCtnr ;
		}
		/**
		* event source display object
		*/
		public function get evtSrc () : DisplayObject {
			return pMc ;
		}
		/**
		* info-bubble  text
		*/
		public function get txt () : String {
			return pTxt ;
		}
		/**
		* info-bubble frame x pos
		*/
		public function get x () : Number {
			return pX ;
		}
		/**
		* info-bubble frame y pos
		*/
		public function get y () : Number {
			return pY ;
		}
		/**
		* info-bubble frame width
		*/
		public function get width () : Number {
			return pW ;
		}
		/**
		* info-bubble frame height
		*/
		public function get height () : Number {
			return pH ;
		}
		/**
		* Delay between rollOver on source and info-bubble display
		*/
		public function get delay () : Number {
			return pDelay ;
		}
		/**
		* Tween duration when info-bubble appear
		*/
		public function get delayTween () : Number {
			return pDelayTween ;
		}
		/**
		* info-bubble frame backGround alpha
		*/
		public function get alpha () : Number {
			return pAlpha ;
		}
		/**
		* info-bubble frame sprite
		*/
		public function get target () : SpriteX {
			return pTarget ;
		}
		/**
		* info-bubble TextField
		*/
		public function get txtFld () : TextFieldX {
			return pTxtFld ;
		}
		//
		//
		/**
		* Display bubble info
		* @param	vX	pos x
		* @param	vY	pos y
		*/
		public function display(vX:Number = NaN, vY:Number = NaN) {	
			var e:Number;
			if (pTxt == null || pTxt == "") pTxt = "No info/Pas d'info !";
			if (isNaN(vX)) vX=pX; 
			if (isNaN(vY)) vY=pY;
			var ctnr:DisplayObjectContainer=pCtnr;
			if (isNaN(vX)) { vX=pMc.x; ctnr=pMc.parent; } 
			if (isNaN(vY)) { vY=pMc.y-pMc.height-8;	ctnr=pMc.parent; } 
			pTarget = new SpriteX(ctnr,0,0,1,"target"); 
			if (!pNoSwap) pTarget.swapDepthX(pMc);			
			pTarget.x = vX;pTarget.y = vY;
			var w:Number=Math.max(displayText(),pW);
 			removeText();		 	 	  
			new RoundRect (pTarget,0,0,w,pH,0xddeeff,pAlpha,0x205050,pAlpha*.5,12) ;
			displayText();
			var s=0; var d=pDelayTween; e=1 ; pTarget.alpha=0;
			new BasicTween(pTarget,"alpha",e,d);
			e=vX; pTarget.x=e-30;
			new BasicTween(pTarget,"x",e,d);
			e=vY; pTarget.y=e+20;
			new BasicTween(pTarget,"y",e,d);
			dispatchEvt("onDisplayBubble",{target:pTarget});	
		}
		/**
		* @private
		*/
		public function doIt() {
			fStart(new MouseEvent(MouseEvent.MOUSE_OVER));
			return this;
		}
		/**
		* @private
		*/
		public function stopIt() {
			fStop(new MouseEvent(MouseEvent.MOUSE_OUT));
			return this;
		}
		/**
		* Force to move info-bubble
		* @param	vX	x pos
		* @param	vY	y pos
		*/
		public function moveTo(vX:Number,vY:Number) {
			new BasicTween(pTarget,"x",vX,0.5); 
			new BasicTween(pTarget,"y",vY,0.5); 
		}
		/**
		* Force to remove bubble without disable event rollOver source
		*/
		public function remove() {				
			if (pDel!=null) pDel.removeEventListener("onDelay",fDisplay);		
			if (pTarget!=null) pTarget.name=pTarget.remove();
			//delete pBg ;
			if (pTxtFld!=null) pTxtFld=pTxtFld.remove(); 
			dispatchEvt("onRemove");
		}
		/**
		* Force to remove and disable event rollOver source
		*/		
		public function disable() {
			remove();
			pMc.removeEventListener(MouseEvent.MOUSE_OVER,fStart);
			pMc.removeEventListener(MouseEvent.MOUSE_OUT,fStop);
		 }
		 /**
		 * @private
		 * Display text in bubble
		 * @return text width
		 */
		public function displayText() : Number {
			removeText();
			pTxtFld = new TextFieldX  (pTarget,2,2,pW,pH,0x333333,getFont(),10,true,true,{autoSize:TextFieldAutoSize.LEFT,wordWrap:false});
			pTxtFld.setText(pTxt);	
			return pTxtFld.textWidth*1.03;
		}	
		 //------Private/protected method ------ 
		/// 
		/**
		* @private
		*/
		protected function removeText()  {
			if (pTxtFld!=null) pTxtFld=pTxtFld.remove();			
		}
		//
		//
		private function createEvent() {
			pMc.addEventListener(MouseEvent.MOUSE_OVER,fStart);
			pMc.addEventListener(MouseEvent.MOUSE_OUT,fStop);
		}
		 private function fStart(e:MouseEvent) {			
			 pDel=new Delay2(pDelay);
			 pDel.addEventListener("onDelay",fDisplay);
		 }
		 protected function fDisplay(e:EventX) {				 
			//if (pCtnr.stage.displayState!=StageDisplayState.FULL_SCREEN) 
			display();
		 }
		 private function fStop(e:MouseEvent) {
			remove();
		 }
		
			
		
	}
}