﻿/**
* Scroll bar manager
* @author Jean-Michel Delettre (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.ui.scroll {	
	
	import flash.display.* ;
	import flash.events.* ;
	import net.flash_line.as3.util.common.*;
	import net.flash_line.as3.util.graph.RectFill;
	import net.flash_line.as3.util.global.*;
	import net.flash_line.as3.util.event.*;
	import flash.geom.Rectangle;
	//
	public class Lift extends Super {
		// Properties
		/// Input
					var pTarget		: Sprite		; 	// container scrolled
					var pWheelZone	: Sprite		; 	// wheel zone - generaly not visible and cover part of target
					var pButton		: Sprite		; 	// middle button
					var pLine		: Sprite		; 	// invisible line guide
					var pTop		: Sprite		; 	// top button
					var pBottom		: Sprite		; 	// bottom button
					var pDrag		: Boolean		; 	// True if user press and drag  button		
					var pClk		: Clock			; 	// 
					var pClkDrag	: Clock			; 	// 
					var pOy			: Number		; 	// scrolled target origin _y
					
		// Constructor
		public function Lift ( vTarget:Sprite,wheelZone:Sprite,vButton:Sprite,vLine:Sprite,vTop:Sprite,vBottom:Sprite) {
			setClassInfo("Lift",this);
			pTarget=vTarget;	
			pWheelZone=wheelZone;	
			pButton=vButton;
			pLine=vLine;
			pTop=vTop;
			pBottom=vBottom;
			pOy=pTarget.y;					
			if (pButton!=null) {
				pButton.y=pLine.y;
				pButton.height=pLine.height* (pLine.height+pTop.height+pBottom.height) /pTarget.height;
				pButton.height=Math.min(pButton.height,pLine.height);
				new RectFill(pButton.parent,pButton.x,pButton.y,pButton.width,1,0,0,"topClickArea");
				new RectFill(pButton.parent,pButton.x,pButton.y+pButton.height,pButton.width,pLine.height-pButton.height,0,0,"bottomClickArea");
				
			}
			createEventListener();
		}
		
		//------Public method ------ 
		//get/set
		public function get topClickArea () :SpriteX {
			return SpriteX(pButton.parent.getChildByName("topClickArea"));
		}
		public function get bottomClickArea () :SpriteX {
			return SpriteX(pButton.parent.getChildByName("bottomClickArea"));
		}
		//
		public function get scrollY () :Number {
			return pTarget.y;
		}
		public function get barY () :Number  {
			if (pButton!=null) return pButton.y ;
			else return null;
		}
		public function get barHeight () :Number  {
			if (pButton!=null) return pButton.height ;
			else return null;
		}
		public function get scrollState () :Object  {
			var o:Object={};
			o.scrollY=scrollY;
			o.barY=barY;
			//o.barHeight=barHeight;
			return o;
		}
		public function set scrollY (n:Number)  {
			pTarget.y=n;
		}
		public function set barY (n:Number)  {
			if (pButton!=null) pButton.y =n;
		}
		public function set barHeight (n:Number)  {
			if (pButton!=null) pButton.height =n;
		}
		public function set scrollState (o:Object)   {
			scrollY=o.scrollY;
			barY=o.barY;
			//barHeight=o.barHeight;
		}
		// Enable onRelease function
		public function createEventListener () {
			removeButtonEvent();
			if (pButton!=null) {
				pButton.addEventListener(MouseEvent.MOUSE_DOWN ,fDown);
				pButton.addEventListener(MouseEvent.MOUSE_UP,fUp);			
				pButton.parent.parent.addEventListener(MouseEvent.MOUSE_UP,fUp);	
			}
			if ( pTop!=null && pBottom!=null ) {
				pTop.addEventListener(MouseEvent.MOUSE_DOWN ,fTop);
				pBottom.addEventListener(MouseEvent.MOUSE_DOWN,fBottom)
			}
			if (topClickArea!=null && bottomClickArea!=null) {
				topClickArea.addEventListener(MouseEvent.CLICK ,fClickTop);
				bottomClickArea.addEventListener(MouseEvent.CLICK ,fClickBottom)
			}
			// Wheel
			if (pButton!=null) pButton.addEventListener(MouseEvent.MOUSE_WHEEL,doWheel);
			pTarget.addEventListener(MouseEvent.MOUSE_WHEEL,doWheel);
			if (pWheelZone!=null) pWheelZone.addEventListener(MouseEvent.MOUSE_WHEEL,doWheel);
			//createWheelEvent();
		}
		public function reset () {
			pTarget.y=pOy;
		}
		public function resize () {
			if (pButton!=null) {
				pButton.height=pLine.height* (pLine.height+pTop.height+pBottom.height) /pTarget.height;
				pButton.height=Math.min(pButton.height,pLine.height);
				resizeClickArea();
			}
		}
		public function resizeClickArea () {
			topClickArea.height=pButton.y-pLine.y;
			bottomClickArea.y=pButton.y+pButton.height;
			bottomClickArea.height=pLine.height-pButton.height-topClickArea.height;
		}
		public function doWheel (e:MouseEvent) {
			var step:Number=e.delta;
			if (step>0)  {
				fTop(e);
			} else if (step<0) {
				step*=-1;
				fBottom(e);
			} 
		}
		
		public function enable() {		
			removeClock();
			pClk=new Clock(onClock);
		}
		public function disable() {
			removeClock();
		}
		public function removeClock() {
			if (pClk!=null) pClk=pClk.remove();
		}
		public function onClock(e:Event) {	
			doScroll ();
		}
		// Remove events
		public function remove() {
			disable();
			removeButtonEvent();
			pTop.removeEventListener(MouseEvent.MOUSE_DOWN ,fTop);
			pBottom.removeEventListener(MouseEvent.MOUSE_DOWN,fBottom);
			pTarget.removeEventListener(MouseEvent.MOUSE_WHEEL,doWheel);
			if (pWheelZone!=null) pWheelZone.removeEventListener(MouseEvent.MOUSE_WHEEL,doWheel);
			
		}
		protected function doScroll () {			
			if (pButton!=null) {
				var shft = pButton.y - pLine.y ;
				pTarget.y = pOy - (shft * (pTarget.height - pLine.height) / (pLine.height-pButton.height) )  ;
			}
			resizeClickArea();
		}
		protected function fTop(e:Event) {
			jumpTop();
		}
		protected function fBottom(e:Event) {
			jumpBottom();
		}
		protected function fClickTop(e:Event) {
			jumpTop(4);
		}
		protected function fClickBottom(e:Event) {
			jumpBottom(4);
		}
		protected function jumpTop(step:Number=1) {
			if (pButton!=null) {
				pButton.y-=15*step;			
				if (pButton.y<pLine.y) pButton.y=pLine.y;
			}
			doScroll();
		}
		protected function jumpBottom(step:Number=1) {
			if (pButton!=null) {
				pButton.y+=15*step;
				if (pButton.y>(pLine.y+pLine.height-pButton.height)) pButton.y=(pLine.y+pLine.height-pButton.height);
			}
			doScroll();
		}
		
		// ----- Private function --------- //
		private function fDown(e:Event) {
			if (pButton!=null) pButton.startDrag ( false, new Rectangle(pLine.x,pLine.y,0,pLine.height-pButton.height));		
			//All.startDrag(pButton,pLine.x,pLine.y,pLine.x,pLine.y+pLine.height-pButton.height)//pLine.y+
			//enableWheel();
			enable();
		}
		private function fUp(e:Event) {
			if (pButton!=null) pButton.stopDrag ();
			//All.stopDrag(pButton);		
			disable();
		}
		private function removeButtonEvent () {			
			if (pButton!=null) {
				pButton.removeEventListener(MouseEvent.MOUSE_DOWN ,fDown);
				pButton.removeEventListener(MouseEvent.MOUSE_UP,fUp);
				pButton.removeEventListener(MouseEvent.MOUSE_WHEEL,doWheel);
			}
			if (pWheelZone!=null) pWheelZone.removeEventListener(MouseEvent.MOUSE_WHEEL,doWheel);
			
		}
		
		
		
		

		
	}
}