﻿/**
* @author Jean-Michel Delettre (jm.delettre@flash-line.net)
* @link http://www.flash-line.net
*/ 
package net.flash_line.as3.util.xml {
	import net.flash_line.as3.util.event.EventX;
	import net.flash_line.as3.util.io.Xdata ;
	import net.flash_line.as3.util.common.Super ;
	import net.flash_line.as3.util.global.* ;
	/**
     * Dispatched when xml is correctly loaded and parsed
	 * @eventType net.flash_line.as3.util.event.EventX.ON_PARSE_XML
	 * @see net.flash_line.as3.util.common.Super#dispatchEvt()
     */
    [Event( name="onParseXml", type="net.flash_line.as3.util.event.EventX" )]
	/**
     * Dispatched when an error occure during loading
	 * @eventType net.flash_line.as3.util.event.EventX.ON_ERROR_LOAD_XML
	 * @see net.flash_line.as3.util.common.Super#dispatchEvt()
     */
    [Event( name="onErrorLoadXml", type="net.flash_line.as3.util.event.EventX" )]
	/**
	* transfert xml in an AS structured Object
	*/
	public class XmlParser   extends Super  {
/**
* xml file
*/
		private var pFile : String;
		private var pErrMsg : String ;
		private var io:Xdata ;
/**
* Object of Object or Array  of... of properties
* it's the AS structured form obtain from the xml
*/
		private var pTree : Object ;
/**
* constructor
* @param	file Xml file 
* @param 	errMsg	a facultatif error message displayed by XmlParser instead of by the caller
*/
		public function XmlParser(file:String="",errMsg:String=null) {	
			pErrMsg=errMsg;
			if (file!="") pFile=file;
			setClassInfo("XmlParser",this);
		}
/**
* Object of Object or Array of...properties relating to the xml structure 
* <p>tree() start from the inner of the first node </p>
* @return stucture of pTree inner <root>
*/		public function get tree () : Object {
			for (var j:String in pTree) {
				var o:Object=pTree[j]; 
				break;
			}
			return o;
		}
		/**
		* @private
		*/
		public function get root () : Object {return tree}		
		/**
		* @private
		*/
		public function get _tree () : Object {return tree }		
		/**
		* @private
		*/
		public function get  _root () : Object {return tree }	
		/**
		* @private
		*/
		public function get  _ptree () : Object {return pTree }	
		
/**
* Load xml file
*/		
		public function loadXml () {
			if (io==null) {
				io = new Xdata();
			} else {
				removeIoListener();
			}
			io.addEventListener("onLoadXml",fOnLoadXml);
			io.addEventListener("onErrorLoadXml",fOnErrorLoadXml);			
			io.loadXml(pFile,pErrMsg);
		}	
/**
* @private
* display result -used only to test 
* @param	o Tree
* @param	tab space charac. - equal to null during first call
*/
		public function display(o:Object,tab:String) {
			tab+="    ";	
			if (o.propertyIsEnumerable ("arrayName")) {	
				var arr=o[o.arrayName];
				for (var i:Number=0;i<arr.length;i++) {				
					alert("1 "+tab+i);
					display(arr[i],tab);//.inverse()
				}
			} else {
				for (var j:String in o) {			//.inverse()
					if ( (o[j]) is String ) alert("3 "+tab+j+"="+o[j]);
					else {
						alert("2 "+tab+j);
						var o2:Object=o[j];
						display(o2,tab);
					}
					
					
				}
				
			}//
		}
/**
* Public entry to parse only - without load xml file
* @param	xml XML instance
* @return inner tree  - same than get tree()
*/		
		public function parse(xml:XML):Object {			
			var n:Node = new Node(xml) ;
			var m:Model=new Model();							
			pTree={};
			m.toObject(n,pTree);
			return tree;	
		}
/**
* listen loadXml() if okay
* @param	e Generic Event - used to pass parameters from event src
*/			
		private function fOnLoadXml(e:EventX) {		
			removeIoListener();
			//
			parse(e.srcVar.xml);
			dispatchEvt(EventX.ON_PARSE_XML,{tree:tree,instance:this } );
		}
		private function fOnErrorLoadXml(e:EventX) {
			removeIoListener();
			//
			dispatchEvt(EventX.ON_ERROR_LOAD_XML,{instance:this });
		}
		private function removeIoListener() {
			if (io.hasEventListener("onLoadXml")) io.removeEventListener("onLoadXml", fOnLoadXml);
			if (io.hasEventListener("onErrorLoadXml"))  io.removeEventListener("onErrorLoadXml",fOnErrorLoadXml);
		}
		
	/**
	* @private
	*/
		public  function remove() {	
			removeIoListener();
		}
	}
	//End of XmlParser
}